#pragma once

// This file contains autogenerated LazyTensor IR nodes
#include <ATen/core/Formatting.h>
#include <c10/core/ScalarType.h>
#include <c10/util/Optional.h>
#include <torch/csrc/lazy/core/hash.h>
#include <torch/csrc/lazy/core/ir.h>
#include <torch/csrc/lazy/core/shape.h>
#include <vector>
#include "/io/python/torch_mlir/csrc/base_lazy_backend/mlir_node.h"

namespace torch {
namespace lazy {
using at::operator<<;

// kNullValue is used to contribute a static hash value any time
// a node has an Optional<Value> input that is nullopt.  It is important
// to differentiate between HASH(nullopt, something) and HASH(something, nullopt),
// and using kNullValue in the hash function in the order of arguments
// serves this purpose.
static const torch::lazy::Value kNullValue = torch::lazy::Value();

class EmbeddingBag : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::_embedding_bag);
  }

  EmbeddingBag(const torch::lazy::Value& weight, const torch::lazy::Value& indices, const torch::lazy::Value& offsets, const bool& scale_grad_by_freq, const int64_t& mode, const bool& sparse, const c10::optional<torch::lazy::Value>& per_sample_weights, const bool& include_last_offset, const int64_t& padding_idx, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              EmbeddingBag::ClassOpKind(),
              OpList{weight, indices, offsets, per_sample_weights.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 4,
              torch::lazy::MHash(scale_grad_by_freq, mode, sparse, include_last_offset, padding_idx)),
        scale_grad_by_freq(scale_grad_by_freq),
        mode(mode),
        sparse(sparse),
        include_last_offset(include_last_offset),
        padding_idx(padding_idx)
  {
    has_per_sample_weights = !!per_sample_weights;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", scale_grad_by_freq=" << scale_grad_by_freq;
    ss << ", mode=" << mode;
    ss << ", sparse=" << sparse;
    ss << ", include_last_offset=" << include_last_offset;
    ss << ", padding_idx=" << padding_idx;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& weight, const torch::lazy::Value& indices, const torch::lazy::Value& offsets, const bool& scale_grad_by_freq, const int64_t& mode, const bool& sparse, const c10::optional<torch::lazy::Value>& per_sample_weights, const bool& include_last_offset, const int64_t& padding_idx) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(9);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("scale_grad_by_freq", scale_grad_by_freq);
  arguments.emplace_back("mode", mode);
  arguments.emplace_back("sparse", sparse);
  arguments.emplace_back(has_per_sample_weights ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back("include_last_offset", include_last_offset);
  arguments.emplace_back("padding_idx", padding_idx);

          torch::lazy::TorchMlirOpVector _embedding_bag_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(_embedding_bag_out.size(), 4);

          return _embedding_bag_out;
      }


  bool scale_grad_by_freq;
  int64_t mode;
  bool sparse;
  bool include_last_offset;
  int64_t padding_idx;
  bool has_per_sample_weights: 1;

};

class LogSoftmax : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::_log_softmax);
  }

  LogSoftmax(const torch::lazy::Value& self, const int64_t& dim, const bool& half_to_float, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              LogSoftmax::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, half_to_float)),
        dim(dim),
        half_to_float(half_to_float)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", half_to_float=" << half_to_float;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim, const bool& half_to_float) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("half_to_float", half_to_float);

          torch::lazy::TorchMlirOpVector _log_softmax_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(_log_softmax_out.size(), 1);

          return _log_softmax_out;
      }


  int64_t dim;
  bool half_to_float;
  

};

class LogSoftmaxBackwardData : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::_log_softmax_backward_data);
  }

  LogSoftmaxBackwardData(const torch::lazy::Value& grad_output, const torch::lazy::Value& output, const int64_t& dim, const at::ScalarType& input_dtype, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              LogSoftmaxBackwardData::ClassOpKind(),
              OpList{grad_output, output},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, input_dtype)),
        dim(dim),
        input_dtype(input_dtype)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", input_dtype=" << input_dtype;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& output, const int64_t& dim, const at::ScalarType& input_dtype) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("input_dtype", input_dtype);

          torch::lazy::TorchMlirOpVector _log_softmax_backward_data_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(_log_softmax_backward_data_out.size(), 1);

          return _log_softmax_backward_data_out;
      }


  int64_t dim;
  at::ScalarType input_dtype;
  

};

class ReshapeAliasCopy : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::_reshape_alias_copy);
  }

  ReshapeAliasCopy(const torch::lazy::Value& self, const ::std::vector<int64_t>& size, const ::std::vector<int64_t>& stride, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ReshapeAliasCopy::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(size, stride)),
        size(size),
        stride(stride)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", size=" << size;
    ss << ", stride=" << stride;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const ::std::vector<int64_t>& size, const ::std::vector<int64_t>& stride) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("size", size);
  arguments.emplace_back("stride", stride);

          torch::lazy::TorchMlirOpVector _reshape_alias_copy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(_reshape_alias_copy_out.size(), 1);

          return _reshape_alias_copy_out;
      }


  ::std::vector<int64_t> size;
  ::std::vector<int64_t> stride;
  

};

class Softmax : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::_softmax);
  }

  Softmax(const torch::lazy::Value& self, const int64_t& dim, const bool& half_to_float, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Softmax::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, half_to_float)),
        dim(dim),
        half_to_float(half_to_float)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", half_to_float=" << half_to_float;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim, const bool& half_to_float) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("half_to_float", half_to_float);

          torch::lazy::TorchMlirOpVector _softmax_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(_softmax_out.size(), 1);

          return _softmax_out;
      }


  int64_t dim;
  bool half_to_float;
  

};

class SoftmaxBackwardData : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::_softmax_backward_data);
  }

  SoftmaxBackwardData(const torch::lazy::Value& grad_output, const torch::lazy::Value& output, const int64_t& dim, const at::ScalarType& input_dtype, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              SoftmaxBackwardData::ClassOpKind(),
              OpList{grad_output, output},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, input_dtype)),
        dim(dim),
        input_dtype(input_dtype)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", input_dtype=" << input_dtype;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& output, const int64_t& dim, const at::ScalarType& input_dtype) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("input_dtype", input_dtype);

          torch::lazy::TorchMlirOpVector _softmax_backward_data_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(_softmax_backward_data_out.size(), 1);

          return _softmax_backward_data_out;
      }


  int64_t dim;
  at::ScalarType input_dtype;
  

};

class Abs : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::abs);
  }

  Abs(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Abs::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector abs_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(abs_out.size(), 1);

      return abs_out;
  }


  
  

};

class AddTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::add);
  }

  AddTensor(const torch::lazy::Value& self, const torch::lazy::Value& other, const torch::lazy::Value& alpha, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              AddTensor::ClassOpKind(),
              OpList{self, other, alpha},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other, const torch::lazy::Value& alpha) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(1);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
          kwarguments.emplace_back("alpha", loctx->GetOutputOp(operand(i++)));
          torch::lazy::TorchMlirOpVector add_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(add_out.size(), 1);

          return add_out;
      }


  
  

};

class AddScalar : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::add);
  }

  AddScalar(const torch::lazy::Value& self, const torch::lazy::Value& other, const torch::lazy::Value& alpha, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              AddScalar::ClassOpKind(),
              OpList{self, other, alpha},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other, const torch::lazy::Value& alpha) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector add_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(add_out.size(), 1);

          return add_out;
      }


  
  

};

class Addcdiv : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::addcdiv);
  }

  Addcdiv(const torch::lazy::Value& self, const torch::lazy::Value& tensor1, const torch::lazy::Value& tensor2, const torch::lazy::Value& value, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Addcdiv::ClassOpKind(),
              OpList{self, tensor1, tensor2, value},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& tensor1, const torch::lazy::Value& tensor2, const torch::lazy::Value& value) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(1);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
          kwarguments.emplace_back("value", loctx->GetOutputOp(operand(i++)));
          torch::lazy::TorchMlirOpVector addcdiv_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(addcdiv_out.size(), 1);

          return addcdiv_out;
      }


  
  

};

class Addcmul : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::addcmul);
  }

  Addcmul(const torch::lazy::Value& self, const torch::lazy::Value& tensor1, const torch::lazy::Value& tensor2, const torch::lazy::Value& value, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Addcmul::ClassOpKind(),
              OpList{self, tensor1, tensor2, value},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& tensor1, const torch::lazy::Value& tensor2, const torch::lazy::Value& value) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(1);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
          kwarguments.emplace_back("value", loctx->GetOutputOp(operand(i++)));
          torch::lazy::TorchMlirOpVector addcmul_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(addcmul_out.size(), 1);

          return addcmul_out;
      }


  
  

};

class Addmm : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::addmm);
  }

  Addmm(const torch::lazy::Value& self, const torch::lazy::Value& mat1, const torch::lazy::Value& mat2, const torch::lazy::Value& beta, const torch::lazy::Value& alpha, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Addmm::ClassOpKind(),
              OpList{self, mat1, mat2, beta, alpha},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& mat1, const torch::lazy::Value& mat2, const torch::lazy::Value& beta, const torch::lazy::Value& alpha) const {
    return false;
    }

  
          TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
              PRINT_FUNCTION();
              std::vector<torch::jit::NamedValue> arguments;
              std::vector<torch::jit::NamedValue> kwarguments;
              arguments.reserve(3);
              kwarguments.reserve(2);
              size_t i = 0;
              arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
              kwarguments.emplace_back("beta", loctx->GetOutputOp(operand(i++)));
  kwarguments.emplace_back("alpha", loctx->GetOutputOp(operand(i++)));
              torch::lazy::TorchMlirOpVector addmm_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
              TORCH_CHECK_EQ(addmm_out.size(), 1);

              return addmm_out;
          }


  
  

};

class Alias : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::alias);
  }

  Alias(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Alias::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector alias_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(alias_out.size(), 1);

      return alias_out;
  }


  
  

};

class AliasCopy : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::alias_copy);
  }

  AliasCopy(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              AliasCopy::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector alias_copy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(alias_copy_out.size(), 1);

      return alias_copy_out;
  }


  
  

};

class AllDim : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::all);
  }

  AllDim(const torch::lazy::Value& self, const int64_t& dim, const bool& keepdim, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              AllDim::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, keepdim)),
        dim(dim),
        keepdim(keepdim)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", keepdim=" << keepdim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim, const bool& keepdim) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("keepdim", keepdim);

          torch::lazy::TorchMlirOpVector all_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(all_out.size(), 1);

          return all_out;
      }


  int64_t dim;
  bool keepdim;
  

};

class All : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::all);
  }

  All(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              All::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector all_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(all_out.size(), 1);

      return all_out;
  }


  
  

};

class Amax : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::amax);
  }

  Amax(const torch::lazy::Value& self, const ::std::vector<int64_t>& dim, const bool& keepdim, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Amax::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, keepdim)),
        dim(dim),
        keepdim(keepdim)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", keepdim=" << keepdim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const ::std::vector<int64_t>& dim, const bool& keepdim) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("keepdim", keepdim);

          torch::lazy::TorchMlirOpVector amax_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(amax_out.size(), 1);

          return amax_out;
      }


  ::std::vector<int64_t> dim;
  bool keepdim;
  

};

class Amin : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::amin);
  }

  Amin(const torch::lazy::Value& self, const ::std::vector<int64_t>& dim, const bool& keepdim, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Amin::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, keepdim)),
        dim(dim),
        keepdim(keepdim)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", keepdim=" << keepdim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const ::std::vector<int64_t>& dim, const bool& keepdim) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("keepdim", keepdim);

          torch::lazy::TorchMlirOpVector amin_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(amin_out.size(), 1);

          return amin_out;
      }


  ::std::vector<int64_t> dim;
  bool keepdim;
  

};

class AnyDim : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::any);
  }

  AnyDim(const torch::lazy::Value& self, const int64_t& dim, const bool& keepdim, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              AnyDim::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, keepdim)),
        dim(dim),
        keepdim(keepdim)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", keepdim=" << keepdim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim, const bool& keepdim) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("keepdim", keepdim);

          torch::lazy::TorchMlirOpVector any_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(any_out.size(), 1);

          return any_out;
      }


  int64_t dim;
  bool keepdim;
  

};

class Any : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::any);
  }

  Any(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Any::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector any_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(any_out.size(), 1);

      return any_out;
  }


  
  

};

class ArangeStartOut : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::arange);
  }

  ArangeStartOut(const torch::lazy::Value& start, const torch::lazy::Value& end, const torch::lazy::Value& step, const torch::lazy::Value& out, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ArangeStartOut::ClassOpKind(),
              OpList{start, end, step, out},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& start, const torch::lazy::Value& end, const torch::lazy::Value& step, const torch::lazy::Value& out) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(1);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
          kwarguments.emplace_back("out", loctx->GetOutputOp(operand(i++)));
          torch::lazy::TorchMlirOpVector arange_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(arange_out.size(), 1);

          return arange_out;
      }


  
  

};

class Argmax : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::argmax);
  }

  Argmax(const torch::lazy::Value& self, const c10::optional<int64_t>& dim, const bool& keepdim, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Argmax::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, keepdim)),
        dim(dim),
        keepdim(keepdim)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    if (dim.has_value()) {
      ss << ", dim=" << dim.value();
    } else {
      ss << ", dim=null";
    }
    ss << ", keepdim=" << keepdim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const c10::optional<int64_t>& dim, const bool& keepdim) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("keepdim", keepdim);

          torch::lazy::TorchMlirOpVector argmax_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(argmax_out.size(), 1);

          return argmax_out;
      }


  c10::optional<int64_t> dim;
  bool keepdim;
  

};

class Argmin : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::argmin);
  }

  Argmin(const torch::lazy::Value& self, const c10::optional<int64_t>& dim, const bool& keepdim, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Argmin::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, keepdim)),
        dim(dim),
        keepdim(keepdim)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    if (dim.has_value()) {
      ss << ", dim=" << dim.value();
    } else {
      ss << ", dim=null";
    }
    ss << ", keepdim=" << keepdim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const c10::optional<int64_t>& dim, const bool& keepdim) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("keepdim", keepdim);

          torch::lazy::TorchMlirOpVector argmin_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(argmin_out.size(), 1);

          return argmin_out;
      }


  c10::optional<int64_t> dim;
  bool keepdim;
  

};

class AsStridedCopy : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::as_strided_copy);
  }

  AsStridedCopy(const torch::lazy::Value& self, const ::std::vector<int64_t>& size, const ::std::vector<int64_t>& stride, const c10::optional<torch::lazy::Value>& storage_offset, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              AsStridedCopy::ClassOpKind(),
              OpList{self, storage_offset.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(size, stride)),
        size(size),
        stride(stride)
  {
    has_storage_offset = !!storage_offset;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", size=" << size;
    ss << ", stride=" << stride;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const ::std::vector<int64_t>& size, const ::std::vector<int64_t>& stride, const c10::optional<torch::lazy::Value>& storage_offset) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("size", size);
  arguments.emplace_back("stride", stride);
  arguments.emplace_back(has_storage_offset ? loctx->GetOutputOp(operand(i++)) : nullptr);

          torch::lazy::TorchMlirOpVector as_strided_copy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(as_strided_copy_out.size(), 1);

          return as_strided_copy_out;
      }


  ::std::vector<int64_t> size;
  ::std::vector<int64_t> stride;
  bool has_storage_offset: 1;

};

class AsStridedScatter : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::as_strided_scatter);
  }

  AsStridedScatter(const torch::lazy::Value& self, const torch::lazy::Value& src, const ::std::vector<int64_t>& size, const ::std::vector<int64_t>& stride, const c10::optional<torch::lazy::Value>& storage_offset, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              AsStridedScatter::ClassOpKind(),
              OpList{self, src, storage_offset.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(size, stride)),
        size(size),
        stride(stride)
  {
    has_storage_offset = !!storage_offset;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", size=" << size;
    ss << ", stride=" << stride;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& src, const ::std::vector<int64_t>& size, const ::std::vector<int64_t>& stride, const c10::optional<torch::lazy::Value>& storage_offset) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(5);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("size", size);
  arguments.emplace_back("stride", stride);
  arguments.emplace_back(has_storage_offset ? loctx->GetOutputOp(operand(i++)) : nullptr);

          torch::lazy::TorchMlirOpVector as_strided_scatter_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(as_strided_scatter_out.size(), 1);

          return as_strided_scatter_out;
      }


  ::std::vector<int64_t> size;
  ::std::vector<int64_t> stride;
  bool has_storage_offset: 1;

};

class Atan : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::atan);
  }

  Atan(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Atan::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector atan_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(atan_out.size(), 1);

      return atan_out;
  }


  
  

};

class Atan2 : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::atan2);
  }

  Atan2(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Atan2::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector atan2_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(atan2_out.size(), 1);

          return atan2_out;
      }


  
  

};

class AvgPool2d : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::avg_pool2d);
  }

  AvgPool2d(const torch::lazy::Value& self, const ::std::vector<int64_t>& kernel_size, const ::std::vector<int64_t>& stride, const ::std::vector<int64_t>& padding, const bool& ceil_mode, const bool& count_include_pad, const c10::optional<int64_t>& divisor_override, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              AvgPool2d::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(kernel_size, stride, padding, ceil_mode, count_include_pad, divisor_override)),
        kernel_size(kernel_size),
        stride(stride),
        padding(padding),
        ceil_mode(ceil_mode),
        count_include_pad(count_include_pad),
        divisor_override(divisor_override)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", kernel_size=" << kernel_size;
    ss << ", stride=" << stride;
    ss << ", padding=" << padding;
    ss << ", ceil_mode=" << ceil_mode;
    ss << ", count_include_pad=" << count_include_pad;
    if (divisor_override.has_value()) {
      ss << ", divisor_override=" << divisor_override.value();
    } else {
      ss << ", divisor_override=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const ::std::vector<int64_t>& kernel_size, const ::std::vector<int64_t>& stride, const ::std::vector<int64_t>& padding, const bool& ceil_mode, const bool& count_include_pad, const c10::optional<int64_t>& divisor_override) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(7);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("kernel_size", kernel_size);
  arguments.emplace_back("stride", stride);
  arguments.emplace_back("padding", padding);
  arguments.emplace_back("ceil_mode", ceil_mode);
  arguments.emplace_back("count_include_pad", count_include_pad);
  arguments.emplace_back("divisor_override", divisor_override);

          torch::lazy::TorchMlirOpVector avg_pool2d_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(avg_pool2d_out.size(), 1);

          return avg_pool2d_out;
      }


  ::std::vector<int64_t> kernel_size;
  ::std::vector<int64_t> stride;
  ::std::vector<int64_t> padding;
  bool ceil_mode;
  bool count_include_pad;
  c10::optional<int64_t> divisor_override;
  

};

class Baddbmm : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::baddbmm);
  }

  Baddbmm(const torch::lazy::Value& self, const torch::lazy::Value& batch1, const torch::lazy::Value& batch2, const torch::lazy::Value& beta, const torch::lazy::Value& alpha, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Baddbmm::ClassOpKind(),
              OpList{self, batch1, batch2, beta, alpha},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& batch1, const torch::lazy::Value& batch2, const torch::lazy::Value& beta, const torch::lazy::Value& alpha) const {
    return false;
    }

  
          TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
              PRINT_FUNCTION();
              std::vector<torch::jit::NamedValue> arguments;
              std::vector<torch::jit::NamedValue> kwarguments;
              arguments.reserve(3);
              kwarguments.reserve(2);
              size_t i = 0;
              arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
              kwarguments.emplace_back("beta", loctx->GetOutputOp(operand(i++)));
  kwarguments.emplace_back("alpha", loctx->GetOutputOp(operand(i++)));
              torch::lazy::TorchMlirOpVector baddbmm_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
              TORCH_CHECK_EQ(baddbmm_out.size(), 1);

              return baddbmm_out;
          }


  
  

};

class Bernoulli : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::bernoulli);
  }

  Bernoulli(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Bernoulli::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector bernoulli_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(bernoulli_out.size(), 1);

      return bernoulli_out;
  }


  
  

};

class BernoulliTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::bernoulli);
  }

  BernoulliTensor(const torch::lazy::Value& self, const torch::lazy::Value& p, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              BernoulliTensor::ClassOpKind(),
              OpList{self, p},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& p) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector bernoulli_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(bernoulli_out.size(), 1);

          return bernoulli_out;
      }


  
  

};

class BernoulliP : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::bernoulli);
  }

  BernoulliP(const torch::lazy::Value& self, const double& p, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              BernoulliP::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(p)),
        p(p)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", p=" << p;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const double& p) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("p", p);

          torch::lazy::TorchMlirOpVector bernoulli_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(bernoulli_out.size(), 1);

          return bernoulli_out;
      }


  double p;
  

};

class BinaryCrossEntropy : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::binary_cross_entropy);
  }

  BinaryCrossEntropy(const torch::lazy::Value& self, const torch::lazy::Value& target, const c10::optional<torch::lazy::Value>& weight, const int64_t& reduction, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              BinaryCrossEntropy::ClassOpKind(),
              OpList{self, target, weight.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(reduction)),
        reduction(reduction)
  {
    has_weight = !!weight;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", reduction=" << reduction;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& target, const c10::optional<torch::lazy::Value>& weight, const int64_t& reduction) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(has_weight ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back("reduction", reduction);

          torch::lazy::TorchMlirOpVector binary_cross_entropy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(binary_cross_entropy_out.size(), 1);

          return binary_cross_entropy_out;
      }


  int64_t reduction;
  bool has_weight: 1;

};

class BinaryCrossEntropyBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::binary_cross_entropy_backward);
  }

  BinaryCrossEntropyBackward(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const torch::lazy::Value& target, const c10::optional<torch::lazy::Value>& weight, const int64_t& reduction, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              BinaryCrossEntropyBackward::ClassOpKind(),
              OpList{grad_output, self, target, weight.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(reduction)),
        reduction(reduction)
  {
    has_weight = !!weight;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", reduction=" << reduction;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const torch::lazy::Value& target, const c10::optional<torch::lazy::Value>& weight, const int64_t& reduction) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(5);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(has_weight ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back("reduction", reduction);

          torch::lazy::TorchMlirOpVector binary_cross_entropy_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(binary_cross_entropy_backward_out.size(), 1);

          return binary_cross_entropy_backward_out;
      }


  int64_t reduction;
  bool has_weight: 1;

};

class Bincount : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::bincount);
  }

  Bincount(const torch::lazy::Value& self, const c10::optional<torch::lazy::Value>& weights, const int64_t& minlength, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Bincount::ClassOpKind(),
              OpList{self, weights.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(minlength)),
        minlength(minlength)
  {
    has_weights = !!weights;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", minlength=" << minlength;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const c10::optional<torch::lazy::Value>& weights, const int64_t& minlength) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(has_weights ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back("minlength", minlength);

          torch::lazy::TorchMlirOpVector bincount_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(bincount_out.size(), 1);

          return bincount_out;
      }


  int64_t minlength;
  bool has_weights: 1;

};

class BitwiseAndTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::bitwise_and);
  }

  BitwiseAndTensor(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              BitwiseAndTensor::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector bitwise_and_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(bitwise_and_out.size(), 1);

          return bitwise_and_out;
      }


  
  

};

class BitwiseNot : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::bitwise_not);
  }

  BitwiseNot(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              BitwiseNot::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector bitwise_not_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(bitwise_not_out.size(), 1);

      return bitwise_not_out;
  }


  
  

};

class BitwiseOrTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::bitwise_or);
  }

  BitwiseOrTensor(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              BitwiseOrTensor::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector bitwise_or_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(bitwise_or_out.size(), 1);

          return bitwise_or_out;
      }


  
  

};

class BitwiseXorTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::bitwise_xor);
  }

  BitwiseXorTensor(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              BitwiseXorTensor::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector bitwise_xor_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(bitwise_xor_out.size(), 1);

          return bitwise_xor_out;
      }


  
  

};

class Bmm : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::bmm);
  }

  Bmm(const torch::lazy::Value& self, const torch::lazy::Value& mat2, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Bmm::ClassOpKind(),
              OpList{self, mat2},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& mat2) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector bmm_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(bmm_out.size(), 1);

          return bmm_out;
      }


  
  

};

class BucketizeTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::bucketize);
  }

  BucketizeTensor(const torch::lazy::Value& self, const torch::lazy::Value& boundaries, const bool& out_int32, const bool& right, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              BucketizeTensor::ClassOpKind(),
              OpList{self, boundaries},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(out_int32, right)),
        out_int32(out_int32),
        right(right)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", out_int32=" << out_int32;
    ss << ", right=" << right;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& boundaries, const bool& out_int32, const bool& right) const {
    return false;
    }

  
          TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
              PRINT_FUNCTION();
              std::vector<torch::jit::NamedValue> arguments;
              std::vector<torch::jit::NamedValue> kwarguments;
              arguments.reserve(2);
              kwarguments.reserve(2);
              size_t i = 0;
              arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
              kwarguments.emplace_back("out_int32", out_int32);
  kwarguments.emplace_back("right", right);
              torch::lazy::TorchMlirOpVector bucketize_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
              TORCH_CHECK_EQ(bucketize_out.size(), 1);

              return bucketize_out;
          }


  bool out_int32;
  bool right;
  

};

class Cat : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::cat);
  }

  Cat(const torch::lazy::Value& tensors, const int64_t& dim, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Cat::ClassOpKind(),
              OpList{tensors},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim)),
        dim(dim)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& tensors, const int64_t& dim) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);

          torch::lazy::TorchMlirOpVector cat_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(cat_out.size(), 1);

          return cat_out;
      }


  int64_t dim;
  

};

class Ceil : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::ceil);
  }

  Ceil(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Ceil::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector ceil_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(ceil_out.size(), 1);

      return ceil_out;
  }


  
  

};

class Clamp : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::clamp);
  }

  Clamp(const torch::lazy::Value& self, const c10::optional<torch::lazy::Value>& min, const c10::optional<torch::lazy::Value>& max, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Clamp::ClassOpKind(),
              OpList{self, min.value_or(kNullValue), max.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    has_min = !!min;
    has_max = !!max;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const c10::optional<torch::lazy::Value>& min, const c10::optional<torch::lazy::Value>& max) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(has_min ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back(has_max ? loctx->GetOutputOp(operand(i++)) : nullptr);

          torch::lazy::TorchMlirOpVector clamp_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(clamp_out.size(), 1);

          return clamp_out;
      }


  
  bool has_min: 1;
  bool has_max: 1;

};

class ClampTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::clamp);
  }

  ClampTensor(const torch::lazy::Value& self, const c10::optional<torch::lazy::Value>& min, const c10::optional<torch::lazy::Value>& max, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ClampTensor::ClassOpKind(),
              OpList{self, min.value_or(kNullValue), max.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    has_min = !!min;
    has_max = !!max;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const c10::optional<torch::lazy::Value>& min, const c10::optional<torch::lazy::Value>& max) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(has_min ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back(has_max ? loctx->GetOutputOp(operand(i++)) : nullptr);

          torch::lazy::TorchMlirOpVector clamp_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(clamp_out.size(), 1);

          return clamp_out;
      }


  
  bool has_min: 1;
  bool has_max: 1;

};

class ClampMax : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::clamp_max);
  }

  ClampMax(const torch::lazy::Value& self, const torch::lazy::Value& max, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ClampMax::ClassOpKind(),
              OpList{self, max},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& max) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector clamp_max_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(clamp_max_out.size(), 1);

          return clamp_max_out;
      }


  
  

};

class ClampMaxTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::clamp_max);
  }

  ClampMaxTensor(const torch::lazy::Value& self, const torch::lazy::Value& max, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ClampMaxTensor::ClassOpKind(),
              OpList{self, max},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& max) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector clamp_max_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(clamp_max_out.size(), 1);

          return clamp_max_out;
      }


  
  

};

class ClampMin : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::clamp_min);
  }

  ClampMin(const torch::lazy::Value& self, const torch::lazy::Value& min, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ClampMin::ClassOpKind(),
              OpList{self, min},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& min) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector clamp_min_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(clamp_min_out.size(), 1);

          return clamp_min_out;
      }


  
  

};

class ClampMinTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::clamp_min);
  }

  ClampMinTensor(const torch::lazy::Value& self, const torch::lazy::Value& min, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ClampMinTensor::ClassOpKind(),
              OpList{self, min},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& min) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector clamp_min_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(clamp_min_out.size(), 1);

          return clamp_min_out;
      }


  
  

};

class ConstantPadNd : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::constant_pad_nd);
  }

  ConstantPadNd(const torch::lazy::Value& self, const ::std::vector<int64_t>& pad, const torch::lazy::Value& value, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ConstantPadNd::ClassOpKind(),
              OpList{self, value},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(pad)),
        pad(pad)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", pad=" << pad;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const ::std::vector<int64_t>& pad, const torch::lazy::Value& value) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("pad", pad);
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector constant_pad_nd_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(constant_pad_nd_out.size(), 1);

          return constant_pad_nd_out;
      }


  ::std::vector<int64_t> pad;
  

};

class Convolution : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::convolution);
  }

  Convolution(const torch::lazy::Value& input, const torch::lazy::Value& weight, const c10::optional<torch::lazy::Value>& bias, const ::std::vector<int64_t>& stride, const ::std::vector<int64_t>& padding, const ::std::vector<int64_t>& dilation, const bool& transposed, const ::std::vector<int64_t>& output_padding, const int64_t& groups, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Convolution::ClassOpKind(),
              OpList{input, weight, bias.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(stride, padding, dilation, transposed, output_padding, groups)),
        stride(stride),
        padding(padding),
        dilation(dilation),
        transposed(transposed),
        output_padding(output_padding),
        groups(groups)
  {
    has_bias = !!bias;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", stride=" << stride;
    ss << ", padding=" << padding;
    ss << ", dilation=" << dilation;
    ss << ", transposed=" << transposed;
    ss << ", output_padding=" << output_padding;
    ss << ", groups=" << groups;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& input, const torch::lazy::Value& weight, const c10::optional<torch::lazy::Value>& bias, const ::std::vector<int64_t>& stride, const ::std::vector<int64_t>& padding, const ::std::vector<int64_t>& dilation, const bool& transposed, const ::std::vector<int64_t>& output_padding, const int64_t& groups) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(9);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(has_bias ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back("stride", stride);
  arguments.emplace_back("padding", padding);
  arguments.emplace_back("dilation", dilation);
  arguments.emplace_back("transposed", transposed);
  arguments.emplace_back("output_padding", output_padding);
  arguments.emplace_back("groups", groups);

          torch::lazy::TorchMlirOpVector convolution_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(convolution_out.size(), 1);

          return convolution_out;
      }


  ::std::vector<int64_t> stride;
  ::std::vector<int64_t> padding;
  ::std::vector<int64_t> dilation;
  bool transposed;
  ::std::vector<int64_t> output_padding;
  int64_t groups;
  bool has_bias: 1;

};

class ConvolutionBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::convolution_backward);
  }

  ConvolutionBackward(const torch::lazy::Value& grad_output, const torch::lazy::Value& input, const torch::lazy::Value& weight, const c10::optional<::std::vector<int64_t>>& bias_sizes, const ::std::vector<int64_t>& stride, const ::std::vector<int64_t>& padding, const ::std::vector<int64_t>& dilation, const bool& transposed, const ::std::vector<int64_t>& output_padding, const int64_t& groups, const ::std::vector<bool>& output_mask, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ConvolutionBackward::ClassOpKind(),
              OpList{grad_output, input, weight},
              std::move(shapes),
              /* num_outputs */ 3,
              torch::lazy::MHash(bias_sizes, stride, padding, dilation, transposed, output_padding, groups, output_mask)),
        bias_sizes(bias_sizes),
        stride(stride),
        padding(padding),
        dilation(dilation),
        transposed(transposed),
        output_padding(output_padding),
        groups(groups),
        output_mask(output_mask)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    if (bias_sizes.has_value()) {
      ss << ", bias_sizes=" << bias_sizes.value();
    } else {
      ss << ", bias_sizes=null";
    }
    ss << ", stride=" << stride;
    ss << ", padding=" << padding;
    ss << ", dilation=" << dilation;
    ss << ", transposed=" << transposed;
    ss << ", output_padding=" << output_padding;
    ss << ", groups=" << groups;
    ss << ", output_mask=" << output_mask;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& input, const torch::lazy::Value& weight, const c10::optional<::std::vector<int64_t>>& bias_sizes, const ::std::vector<int64_t>& stride, const ::std::vector<int64_t>& padding, const ::std::vector<int64_t>& dilation, const bool& transposed, const ::std::vector<int64_t>& output_padding, const int64_t& groups, const ::std::vector<bool>& output_mask) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(11);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("bias_sizes", bias_sizes);
  arguments.emplace_back("stride", stride);
  arguments.emplace_back("padding", padding);
  arguments.emplace_back("dilation", dilation);
  arguments.emplace_back("transposed", transposed);
  arguments.emplace_back("output_padding", output_padding);
  arguments.emplace_back("groups", groups);
  arguments.emplace_back("output_mask", output_mask);

          torch::lazy::TorchMlirOpVector convolution_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(convolution_backward_out.size(), 3);

          return convolution_backward_out;
      }


  c10::optional<::std::vector<int64_t>> bias_sizes;
  ::std::vector<int64_t> stride;
  ::std::vector<int64_t> padding;
  ::std::vector<int64_t> dilation;
  bool transposed;
  ::std::vector<int64_t> output_padding;
  int64_t groups;
  ::std::vector<bool> output_mask;
  

};

class Copy : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::copy);
  }

  Copy(const torch::lazy::Value& self, const torch::lazy::Value& src, const bool& non_blocking, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Copy::ClassOpKind(),
              OpList{self, src},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(non_blocking)),
        non_blocking(non_blocking)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", non_blocking=" << non_blocking;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& src, const bool& non_blocking) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("non_blocking", non_blocking);

          torch::lazy::TorchMlirOpVector copy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(copy_out.size(), 1);

          return copy_out;
      }


  bool non_blocking;
  

};

class Cos : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::cos);
  }

  Cos(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Cos::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector cos_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(cos_out.size(), 1);

      return cos_out;
  }


  
  

};

class Cumsum : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::cumsum);
  }

  Cumsum(const torch::lazy::Value& self, const int64_t& dim, const c10::optional<at::ScalarType>& dtype, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Cumsum::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, dtype)),
        dim(dim),
        dtype(dtype)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    if (dtype.has_value()) {
      ss << ", dtype=" << dtype.value();
    } else {
      ss << ", dtype=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim, const c10::optional<at::ScalarType>& dtype) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(1);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
          kwarguments.emplace_back("dtype", dtype);
          torch::lazy::TorchMlirOpVector cumsum_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(cumsum_out.size(), 1);

          return cumsum_out;
      }


  int64_t dim;
  c10::optional<at::ScalarType> dtype;
  

};

class DetachCopy : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::detach_copy);
  }

  DetachCopy(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              DetachCopy::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector detach_copy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(detach_copy_out.size(), 1);

      return detach_copy_out;
  }


  
  

};

class DiagonalCopy : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::diagonal_copy);
  }

  DiagonalCopy(const torch::lazy::Value& self, const int64_t& offset, const int64_t& dim1, const int64_t& dim2, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              DiagonalCopy::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(offset, dim1, dim2)),
        offset(offset),
        dim1(dim1),
        dim2(dim2)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", offset=" << offset;
    ss << ", dim1=" << dim1;
    ss << ", dim2=" << dim2;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& offset, const int64_t& dim1, const int64_t& dim2) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("offset", offset);
  arguments.emplace_back("dim1", dim1);
  arguments.emplace_back("dim2", dim2);

          torch::lazy::TorchMlirOpVector diagonal_copy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(diagonal_copy_out.size(), 1);

          return diagonal_copy_out;
      }


  int64_t offset;
  int64_t dim1;
  int64_t dim2;
  

};

class DiagonalScatter : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::diagonal_scatter);
  }

  DiagonalScatter(const torch::lazy::Value& self, const torch::lazy::Value& src, const int64_t& offset, const int64_t& dim1, const int64_t& dim2, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              DiagonalScatter::ClassOpKind(),
              OpList{self, src},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(offset, dim1, dim2)),
        offset(offset),
        dim1(dim1),
        dim2(dim2)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", offset=" << offset;
    ss << ", dim1=" << dim1;
    ss << ", dim2=" << dim2;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& src, const int64_t& offset, const int64_t& dim1, const int64_t& dim2) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(5);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("offset", offset);
  arguments.emplace_back("dim1", dim1);
  arguments.emplace_back("dim2", dim2);

          torch::lazy::TorchMlirOpVector diagonal_scatter_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(diagonal_scatter_out.size(), 1);

          return diagonal_scatter_out;
      }


  int64_t offset;
  int64_t dim1;
  int64_t dim2;
  

};

class DivTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::div);
  }

  DivTensor(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              DivTensor::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector div_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(div_out.size(), 1);

          return div_out;
      }


  
  

};

class DivTensorMode : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::div);
  }

  DivTensorMode(const torch::lazy::Value& self, const torch::lazy::Value& other, const c10::optional<c10::string_view>& rounding_mode, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              DivTensorMode::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(rounding_mode)),
        rounding_mode(rounding_mode.has_value() ? c10::make_optional(std::string(*rounding_mode)) : c10::nullopt)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    if (rounding_mode.has_value()) {
      ss << ", rounding_mode=" << rounding_mode.value();
    } else {
      ss << ", rounding_mode=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other, const c10::optional<c10::string_view>& rounding_mode) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(1);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
          kwarguments.emplace_back("rounding_mode", rounding_mode);
          torch::lazy::TorchMlirOpVector div_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(div_out.size(), 1);

          return div_out;
      }


  c10::optional<std::string> rounding_mode;
  

};

class DivScalar : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::div);
  }

  DivScalar(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              DivScalar::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector div_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(div_out.size(), 1);

          return div_out;
      }


  
  

};

class Elu : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::elu);
  }

  Elu(const torch::lazy::Value& self, const torch::lazy::Value& alpha, const torch::lazy::Value& scale, const torch::lazy::Value& input_scale, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Elu::ClassOpKind(),
              OpList{self, alpha, scale, input_scale},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& alpha, const torch::lazy::Value& scale, const torch::lazy::Value& input_scale) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector elu_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(elu_out.size(), 1);

          return elu_out;
      }


  
  

};

class EluBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::elu_backward);
  }

  EluBackward(const torch::lazy::Value& grad_output, const torch::lazy::Value& alpha, const torch::lazy::Value& scale, const torch::lazy::Value& input_scale, const bool& is_result, const torch::lazy::Value& self_or_result, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              EluBackward::ClassOpKind(),
              OpList{grad_output, alpha, scale, input_scale, self_or_result},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(is_result)),
        is_result(is_result)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", is_result=" << is_result;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& alpha, const torch::lazy::Value& scale, const torch::lazy::Value& input_scale, const bool& is_result, const torch::lazy::Value& self_or_result) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(6);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("is_result", is_result);
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector elu_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(elu_backward_out.size(), 1);

          return elu_backward_out;
      }


  bool is_result;
  

};

class Embedding : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::embedding);
  }

  Embedding(const torch::lazy::Value& weight, const torch::lazy::Value& indices, const int64_t& padding_idx, const bool& scale_grad_by_freq, const bool& sparse, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Embedding::ClassOpKind(),
              OpList{weight, indices},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(padding_idx, scale_grad_by_freq, sparse)),
        padding_idx(padding_idx),
        scale_grad_by_freq(scale_grad_by_freq),
        sparse(sparse)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", padding_idx=" << padding_idx;
    ss << ", scale_grad_by_freq=" << scale_grad_by_freq;
    ss << ", sparse=" << sparse;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& weight, const torch::lazy::Value& indices, const int64_t& padding_idx, const bool& scale_grad_by_freq, const bool& sparse) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(5);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("padding_idx", padding_idx);
  arguments.emplace_back("scale_grad_by_freq", scale_grad_by_freq);
  arguments.emplace_back("sparse", sparse);

          torch::lazy::TorchMlirOpVector embedding_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(embedding_out.size(), 1);

          return embedding_out;
      }


  int64_t padding_idx;
  bool scale_grad_by_freq;
  bool sparse;
  

};

class EmbeddingDenseBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::embedding_dense_backward);
  }

  EmbeddingDenseBackward(const torch::lazy::Value& grad_output, const torch::lazy::Value& indices, const int64_t& num_weights, const int64_t& padding_idx, const bool& scale_grad_by_freq, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              EmbeddingDenseBackward::ClassOpKind(),
              OpList{grad_output, indices},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(num_weights, padding_idx, scale_grad_by_freq)),
        num_weights(num_weights),
        padding_idx(padding_idx),
        scale_grad_by_freq(scale_grad_by_freq)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", num_weights=" << num_weights;
    ss << ", padding_idx=" << padding_idx;
    ss << ", scale_grad_by_freq=" << scale_grad_by_freq;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& indices, const int64_t& num_weights, const int64_t& padding_idx, const bool& scale_grad_by_freq) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(5);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("num_weights", num_weights);
  arguments.emplace_back("padding_idx", padding_idx);
  arguments.emplace_back("scale_grad_by_freq", scale_grad_by_freq);

          torch::lazy::TorchMlirOpVector embedding_dense_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(embedding_dense_backward_out.size(), 1);

          return embedding_dense_backward_out;
      }


  int64_t num_weights;
  int64_t padding_idx;
  bool scale_grad_by_freq;
  

};

class EqScalar : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::eq);
  }

  EqScalar(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              EqScalar::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector eq_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(eq_out.size(), 1);

          return eq_out;
      }


  
  

};

class EqTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::eq);
  }

  EqTensor(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              EqTensor::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector eq_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(eq_out.size(), 1);

          return eq_out;
      }


  
  

};

class Erf : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::erf);
  }

  Erf(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Erf::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector erf_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(erf_out.size(), 1);

      return erf_out;
  }


  
  

};

class Erfinv : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::erfinv);
  }

  Erfinv(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Erfinv::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector erfinv_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(erfinv_out.size(), 1);

      return erfinv_out;
  }


  
  

};

class Exp : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::exp);
  }

  Exp(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Exp::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector exp_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(exp_out.size(), 1);

      return exp_out;
  }


  
  

};

class ExpandCopy : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::expand_copy);
  }

  ExpandCopy(const torch::lazy::Value& self, const ::std::vector<int64_t>& size, const bool& implicit, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ExpandCopy::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(size, implicit)),
        size(size),
        implicit(implicit)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", size=" << size;
    ss << ", implicit=" << implicit;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const ::std::vector<int64_t>& size, const bool& implicit) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(1);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("size", size);
          kwarguments.emplace_back("implicit", implicit);
          torch::lazy::TorchMlirOpVector expand_copy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(expand_copy_out.size(), 1);

          return expand_copy_out;
      }


  ::std::vector<int64_t> size;
  bool implicit;
  

};

class Expm1 : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::expm1);
  }

  Expm1(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Expm1::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector expm1_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(expm1_out.size(), 1);

      return expm1_out;
  }


  
  

};

class Eye : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::eye);
  }

  Eye(const int64_t& n, const c10::optional<at::ScalarType>& dtype, const c10::optional<at::Layout>& layout, const c10::optional<at::Device>& device, const c10::optional<bool>& pin_memory, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Eye::ClassOpKind(),
              OpList{},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(n, dtype, layout, device, pin_memory)),
        n(n),
        dtype(dtype),
        layout(layout),
        device(device),
        pin_memory(pin_memory)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", n=" << n;
    if (dtype.has_value()) {
      ss << ", dtype=" << dtype.value();
    } else {
      ss << ", dtype=null";
    }
    if (layout.has_value()) {
      ss << ", layout=" << layout.value();
    } else {
      ss << ", layout=null";
    }
    if (device.has_value()) {
      ss << ", device=" << device.value();
    } else {
      ss << ", device=null";
    }
    if (pin_memory.has_value()) {
      ss << ", pin_memory=" << pin_memory.value();
    } else {
      ss << ", pin_memory=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const int64_t& n, const c10::optional<at::ScalarType>& dtype, const c10::optional<at::Layout>& layout, const c10::optional<at::Device>& device, const c10::optional<bool>& pin_memory) const {
    return false;
    }

  
          TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
              PRINT_FUNCTION();
              std::vector<torch::jit::NamedValue> arguments;
              std::vector<torch::jit::NamedValue> kwarguments;
              arguments.reserve(1);
              kwarguments.reserve(4);

              arguments.emplace_back("n", n);
              kwarguments.emplace_back("dtype", dtype);
  kwarguments.emplace_back("layout", layout);
  kwarguments.emplace_back("device", device);
  kwarguments.emplace_back("pin_memory", pin_memory);
              torch::lazy::TorchMlirOpVector eye_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
              TORCH_CHECK_EQ(eye_out.size(), 1);

              return eye_out;
          }


  int64_t n;
  c10::optional<at::ScalarType> dtype;
  c10::optional<at::Layout> layout;
  c10::optional<at::Device> device;
  c10::optional<bool> pin_memory;
  

};

class EyeM : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::eye);
  }

  EyeM(const int64_t& n, const int64_t& m, const c10::optional<at::ScalarType>& dtype, const c10::optional<at::Layout>& layout, const c10::optional<at::Device>& device, const c10::optional<bool>& pin_memory, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              EyeM::ClassOpKind(),
              OpList{},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(n, m, dtype, layout, device, pin_memory)),
        n(n),
        m(m),
        dtype(dtype),
        layout(layout),
        device(device),
        pin_memory(pin_memory)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", n=" << n;
    ss << ", m=" << m;
    if (dtype.has_value()) {
      ss << ", dtype=" << dtype.value();
    } else {
      ss << ", dtype=null";
    }
    if (layout.has_value()) {
      ss << ", layout=" << layout.value();
    } else {
      ss << ", layout=null";
    }
    if (device.has_value()) {
      ss << ", device=" << device.value();
    } else {
      ss << ", device=null";
    }
    if (pin_memory.has_value()) {
      ss << ", pin_memory=" << pin_memory.value();
    } else {
      ss << ", pin_memory=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const int64_t& n, const int64_t& m, const c10::optional<at::ScalarType>& dtype, const c10::optional<at::Layout>& layout, const c10::optional<at::Device>& device, const c10::optional<bool>& pin_memory) const {
    return false;
    }

  
          TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
              PRINT_FUNCTION();
              std::vector<torch::jit::NamedValue> arguments;
              std::vector<torch::jit::NamedValue> kwarguments;
              arguments.reserve(2);
              kwarguments.reserve(4);

              arguments.emplace_back("n", n);
      arguments.emplace_back("m", m);
              kwarguments.emplace_back("dtype", dtype);
  kwarguments.emplace_back("layout", layout);
  kwarguments.emplace_back("device", device);
  kwarguments.emplace_back("pin_memory", pin_memory);
              torch::lazy::TorchMlirOpVector eye_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
              TORCH_CHECK_EQ(eye_out.size(), 1);

              return eye_out;
          }


  int64_t n;
  int64_t m;
  c10::optional<at::ScalarType> dtype;
  c10::optional<at::Layout> layout;
  c10::optional<at::Device> device;
  c10::optional<bool> pin_memory;
  

};

class FillTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::fill);
  }

  FillTensor(const torch::lazy::Value& self, const torch::lazy::Value& value, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              FillTensor::ClassOpKind(),
              OpList{self, value},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& value) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector fill_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(fill_out.size(), 1);

          return fill_out;
      }


  
  

};

class Flip : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::flip);
  }

  Flip(const torch::lazy::Value& self, const ::std::vector<int64_t>& dims, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Flip::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dims)),
        dims(dims)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dims=" << dims;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const ::std::vector<int64_t>& dims) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dims", dims);

          torch::lazy::TorchMlirOpVector flip_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(flip_out.size(), 1);

          return flip_out;
      }


  ::std::vector<int64_t> dims;
  

};

class Floor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::floor);
  }

  Floor(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Floor::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector floor_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(floor_out.size(), 1);

      return floor_out;
  }


  
  

};

class FloorDivide : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::floor_divide);
  }

  FloorDivide(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              FloorDivide::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector floor_divide_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(floor_divide_out.size(), 1);

          return floor_divide_out;
      }


  
  

};

class FmodScalar : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::fmod);
  }

  FmodScalar(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              FmodScalar::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector fmod_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(fmod_out.size(), 1);

          return fmod_out;
      }


  
  

};

class FmodTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::fmod);
  }

  FmodTensor(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              FmodTensor::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector fmod_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(fmod_out.size(), 1);

          return fmod_out;
      }


  
  

};

class Full : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::full);
  }

  Full(const ::std::vector<int64_t>& size, const torch::lazy::Value& fill_value, const c10::optional<at::ScalarType>& dtype, const c10::optional<at::Layout>& layout, const c10::optional<at::Device>& device, const c10::optional<bool>& pin_memory, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Full::ClassOpKind(),
              OpList{fill_value},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(size, dtype, layout, device, pin_memory)),
        size(size),
        dtype(dtype),
        layout(layout),
        device(device),
        pin_memory(pin_memory)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", size=" << size;
    if (dtype.has_value()) {
      ss << ", dtype=" << dtype.value();
    } else {
      ss << ", dtype=null";
    }
    if (layout.has_value()) {
      ss << ", layout=" << layout.value();
    } else {
      ss << ", layout=null";
    }
    if (device.has_value()) {
      ss << ", device=" << device.value();
    } else {
      ss << ", device=null";
    }
    if (pin_memory.has_value()) {
      ss << ", pin_memory=" << pin_memory.value();
    } else {
      ss << ", pin_memory=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const ::std::vector<int64_t>& size, const torch::lazy::Value& fill_value, const c10::optional<at::ScalarType>& dtype, const c10::optional<at::Layout>& layout, const c10::optional<at::Device>& device, const c10::optional<bool>& pin_memory) const {
    return false;
    }

  
          TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
              PRINT_FUNCTION();
              std::vector<torch::jit::NamedValue> arguments;
              std::vector<torch::jit::NamedValue> kwarguments;
              arguments.reserve(2);
              kwarguments.reserve(4);
              size_t i = 0;
              arguments.emplace_back("size", size);
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
              kwarguments.emplace_back("dtype", dtype);
  kwarguments.emplace_back("layout", layout);
  kwarguments.emplace_back("device", device);
  kwarguments.emplace_back("pin_memory", pin_memory);
              torch::lazy::TorchMlirOpVector full_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
              TORCH_CHECK_EQ(full_out.size(), 1);

              return full_out;
          }


  ::std::vector<int64_t> size;
  c10::optional<at::ScalarType> dtype;
  c10::optional<at::Layout> layout;
  c10::optional<at::Device> device;
  c10::optional<bool> pin_memory;
  

};

class Gather : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::gather);
  }

  Gather(const torch::lazy::Value& self, const int64_t& dim, const torch::lazy::Value& index, const bool& sparse_grad, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Gather::ClassOpKind(),
              OpList{self, index},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, sparse_grad)),
        dim(dim),
        sparse_grad(sparse_grad)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", sparse_grad=" << sparse_grad;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim, const torch::lazy::Value& index, const bool& sparse_grad) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(1);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
          kwarguments.emplace_back("sparse_grad", sparse_grad);
          torch::lazy::TorchMlirOpVector gather_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(gather_out.size(), 1);

          return gather_out;
      }


  int64_t dim;
  bool sparse_grad;
  

};

class GeScalar : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::ge);
  }

  GeScalar(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              GeScalar::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector ge_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(ge_out.size(), 1);

          return ge_out;
      }


  
  

};

class GeTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::ge);
  }

  GeTensor(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              GeTensor::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector ge_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(ge_out.size(), 1);

          return ge_out;
      }


  
  

};

class Gelu : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::gelu);
  }

  Gelu(const torch::lazy::Value& self, const c10::string_view& approximate, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Gelu::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(approximate)),
        approximate(approximate)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", approximate=" << approximate;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const c10::string_view& approximate) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(1);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
      kwarguments.emplace_back("approximate", approximate);
      torch::lazy::TorchMlirOpVector gelu_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(gelu_out.size(), 1);

      return gelu_out;
  }


  std::string approximate;
  

};

class GeluBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::gelu_backward);
  }

  GeluBackward(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const c10::string_view& approximate, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              GeluBackward::ClassOpKind(),
              OpList{grad_output, self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(approximate)),
        approximate(approximate)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", approximate=" << approximate;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const c10::string_view& approximate) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(1);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
          kwarguments.emplace_back("approximate", approximate);
          torch::lazy::TorchMlirOpVector gelu_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(gelu_backward_out.size(), 1);

          return gelu_backward_out;
      }


  std::string approximate;
  

};

class GtScalar : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::gt);
  }

  GtScalar(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              GtScalar::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector gt_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(gt_out.size(), 1);

          return gt_out;
      }


  
  

};

class GtTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::gt);
  }

  GtTensor(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              GtTensor::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector gt_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(gt_out.size(), 1);

          return gt_out;
      }


  
  

};

class Hardsigmoid : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::hardsigmoid);
  }

  Hardsigmoid(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Hardsigmoid::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector hardsigmoid_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(hardsigmoid_out.size(), 1);

      return hardsigmoid_out;
  }


  
  

};

class Hardswish : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::hardswish);
  }

  Hardswish(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Hardswish::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector hardswish_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(hardswish_out.size(), 1);

      return hardswish_out;
  }


  
  

};

class Hardtanh : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::hardtanh);
  }

  Hardtanh(const torch::lazy::Value& self, const torch::lazy::Value& min_val, const torch::lazy::Value& max_val, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Hardtanh::ClassOpKind(),
              OpList{self, min_val, max_val},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& min_val, const torch::lazy::Value& max_val) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector hardtanh_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(hardtanh_out.size(), 1);

          return hardtanh_out;
      }


  
  

};

class HardtanhBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::hardtanh_backward);
  }

  HardtanhBackward(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const torch::lazy::Value& min_val, const torch::lazy::Value& max_val, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              HardtanhBackward::ClassOpKind(),
              OpList{grad_output, self, min_val, max_val},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const torch::lazy::Value& min_val, const torch::lazy::Value& max_val) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector hardtanh_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(hardtanh_backward_out.size(), 1);

          return hardtanh_backward_out;
      }


  
  

};

class Im2col : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::im2col);
  }

  Im2col(const torch::lazy::Value& self, const ::std::vector<int64_t>& kernel_size, const ::std::vector<int64_t>& dilation, const ::std::vector<int64_t>& padding, const ::std::vector<int64_t>& stride, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Im2col::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(kernel_size, dilation, padding, stride)),
        kernel_size(kernel_size),
        dilation(dilation),
        padding(padding),
        stride(stride)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", kernel_size=" << kernel_size;
    ss << ", dilation=" << dilation;
    ss << ", padding=" << padding;
    ss << ", stride=" << stride;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const ::std::vector<int64_t>& kernel_size, const ::std::vector<int64_t>& dilation, const ::std::vector<int64_t>& padding, const ::std::vector<int64_t>& stride) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(5);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("kernel_size", kernel_size);
  arguments.emplace_back("dilation", dilation);
  arguments.emplace_back("padding", padding);
  arguments.emplace_back("stride", stride);

          torch::lazy::TorchMlirOpVector im2col_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(im2col_out.size(), 1);

          return im2col_out;
      }


  ::std::vector<int64_t> kernel_size;
  ::std::vector<int64_t> dilation;
  ::std::vector<int64_t> padding;
  ::std::vector<int64_t> stride;
  

};

class IndexSelect : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::index_select);
  }

  IndexSelect(const torch::lazy::Value& self, const int64_t& dim, const torch::lazy::Value& index, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              IndexSelect::ClassOpKind(),
              OpList{self, index},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim)),
        dim(dim)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim, const torch::lazy::Value& index) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector index_select_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(index_select_out.size(), 1);

          return index_select_out;
      }


  int64_t dim;
  

};

class Isnan : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::isnan);
  }

  Isnan(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Isnan::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector isnan_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(isnan_out.size(), 1);

      return isnan_out;
  }


  
  

};

class LeScalar : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::le);
  }

  LeScalar(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              LeScalar::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector le_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(le_out.size(), 1);

          return le_out;
      }


  
  

};

class LeTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::le);
  }

  LeTensor(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              LeTensor::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector le_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(le_out.size(), 1);

          return le_out;
      }


  
  

};

class LeakyRelu : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::leaky_relu);
  }

  LeakyRelu(const torch::lazy::Value& self, const torch::lazy::Value& negative_slope, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              LeakyRelu::ClassOpKind(),
              OpList{self, negative_slope},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& negative_slope) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector leaky_relu_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(leaky_relu_out.size(), 1);

          return leaky_relu_out;
      }


  
  

};

class LeakyReluBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::leaky_relu_backward);
  }

  LeakyReluBackward(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const torch::lazy::Value& negative_slope, const bool& self_is_result, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              LeakyReluBackward::ClassOpKind(),
              OpList{grad_output, self, negative_slope},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(self_is_result)),
        self_is_result(self_is_result)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", self_is_result=" << self_is_result;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const torch::lazy::Value& negative_slope, const bool& self_is_result) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("self_is_result", self_is_result);

          torch::lazy::TorchMlirOpVector leaky_relu_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(leaky_relu_backward_out.size(), 1);

          return leaky_relu_backward_out;
      }


  bool self_is_result;
  

};

class LerpTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::lerp);
  }

  LerpTensor(const torch::lazy::Value& self, const torch::lazy::Value& end, const torch::lazy::Value& weight, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              LerpTensor::ClassOpKind(),
              OpList{self, end, weight},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& end, const torch::lazy::Value& weight) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector lerp_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(lerp_out.size(), 1);

          return lerp_out;
      }


  
  

};

class LinalgQr : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::linalg_qr);
  }

  LinalgQr(const torch::lazy::Value& A, const c10::string_view& mode, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              LinalgQr::ClassOpKind(),
              OpList{A},
              std::move(shapes),
              /* num_outputs */ 2,
              torch::lazy::MHash(mode)),
        mode(mode)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", mode=" << mode;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& A, const c10::string_view& mode) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("mode", mode);

          torch::lazy::TorchMlirOpVector linalg_qr_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(linalg_qr_out.size(), 2);

          return linalg_qr_out;
      }


  std::string mode;
  

};

class LinalgVectorNorm : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::linalg_vector_norm);
  }

  LinalgVectorNorm(const torch::lazy::Value& self, const torch::lazy::Value& ord, const c10::optional<::std::vector<int64_t>>& dim, const bool& keepdim, const c10::optional<at::ScalarType>& dtype, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              LinalgVectorNorm::ClassOpKind(),
              OpList{self, ord},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, keepdim, dtype)),
        dim(dim),
        keepdim(keepdim),
        dtype(dtype)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    if (dim.has_value()) {
      ss << ", dim=" << dim.value();
    } else {
      ss << ", dim=null";
    }
    ss << ", keepdim=" << keepdim;
    if (dtype.has_value()) {
      ss << ", dtype=" << dtype.value();
    } else {
      ss << ", dtype=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& ord, const c10::optional<::std::vector<int64_t>>& dim, const bool& keepdim, const c10::optional<at::ScalarType>& dtype) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(1);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("keepdim", keepdim);
          kwarguments.emplace_back("dtype", dtype);
          torch::lazy::TorchMlirOpVector linalg_vector_norm_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(linalg_vector_norm_out.size(), 1);

          return linalg_vector_norm_out;
      }


  c10::optional<::std::vector<int64_t>> dim;
  bool keepdim;
  c10::optional<at::ScalarType> dtype;
  

};

class Log : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::log);
  }

  Log(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Log::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector log_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(log_out.size(), 1);

      return log_out;
  }


  
  

};

class Log1p : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::log1p);
  }

  Log1p(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Log1p::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector log1p_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(log1p_out.size(), 1);

      return log1p_out;
  }


  
  

};

class Log2 : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::log2);
  }

  Log2(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Log2::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector log2_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(log2_out.size(), 1);

      return log2_out;
  }


  
  

};

class LogSigmoidBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::log_sigmoid_backward);
  }

  LogSigmoidBackward(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const torch::lazy::Value& buffer, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              LogSigmoidBackward::ClassOpKind(),
              OpList{grad_output, self, buffer},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const torch::lazy::Value& buffer) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector log_sigmoid_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(log_sigmoid_backward_out.size(), 1);

          return log_sigmoid_backward_out;
      }


  
  

};

class LogSigmoidForward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::log_sigmoid_forward);
  }

  LogSigmoidForward(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              LogSigmoidForward::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 2,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector log_sigmoid_forward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(log_sigmoid_forward_out.size(), 2);

      return log_sigmoid_forward_out;
  }


  
  

};

class LogicalAnd : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::logical_and);
  }

  LogicalAnd(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              LogicalAnd::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector logical_and_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(logical_and_out.size(), 1);

          return logical_and_out;
      }


  
  

};

class LogicalNot : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::logical_not);
  }

  LogicalNot(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              LogicalNot::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector logical_not_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(logical_not_out.size(), 1);

      return logical_not_out;
  }


  
  

};

class LogicalOr : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::logical_or);
  }

  LogicalOr(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              LogicalOr::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector logical_or_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(logical_or_out.size(), 1);

          return logical_or_out;
      }


  
  

};

class LogicalXor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::logical_xor);
  }

  LogicalXor(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              LogicalXor::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector logical_xor_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(logical_xor_out.size(), 1);

          return logical_xor_out;
      }


  
  

};

class Logsumexp : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::logsumexp);
  }

  Logsumexp(const torch::lazy::Value& self, const ::std::vector<int64_t>& dim, const bool& keepdim, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Logsumexp::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, keepdim)),
        dim(dim),
        keepdim(keepdim)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", keepdim=" << keepdim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const ::std::vector<int64_t>& dim, const bool& keepdim) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("keepdim", keepdim);

          torch::lazy::TorchMlirOpVector logsumexp_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(logsumexp_out.size(), 1);

          return logsumexp_out;
      }


  ::std::vector<int64_t> dim;
  bool keepdim;
  

};

class LtScalar : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::lt);
  }

  LtScalar(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              LtScalar::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector lt_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(lt_out.size(), 1);

          return lt_out;
      }


  
  

};

class LtTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::lt);
  }

  LtTensor(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              LtTensor::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector lt_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(lt_out.size(), 1);

          return lt_out;
      }


  
  

};

class MaskedFillScalar : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::masked_fill);
  }

  MaskedFillScalar(const torch::lazy::Value& self, const torch::lazy::Value& mask, const torch::lazy::Value& value, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              MaskedFillScalar::ClassOpKind(),
              OpList{self, mask, value},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& mask, const torch::lazy::Value& value) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector masked_fill_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(masked_fill_out.size(), 1);

          return masked_fill_out;
      }


  
  

};

class MaskedFillTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::masked_fill);
  }

  MaskedFillTensor(const torch::lazy::Value& self, const torch::lazy::Value& mask, const torch::lazy::Value& value, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              MaskedFillTensor::ClassOpKind(),
              OpList{self, mask, value},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& mask, const torch::lazy::Value& value) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector masked_fill_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(masked_fill_out.size(), 1);

          return masked_fill_out;
      }


  
  

};

class MaskedScatter : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::masked_scatter);
  }

  MaskedScatter(const torch::lazy::Value& self, const torch::lazy::Value& mask, const torch::lazy::Value& source, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              MaskedScatter::ClassOpKind(),
              OpList{self, mask, source},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& mask, const torch::lazy::Value& source) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector masked_scatter_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(masked_scatter_out.size(), 1);

          return masked_scatter_out;
      }


  
  

};

class MaskedSelect : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::masked_select);
  }

  MaskedSelect(const torch::lazy::Value& self, const torch::lazy::Value& mask, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              MaskedSelect::ClassOpKind(),
              OpList{self, mask},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& mask) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector masked_select_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(masked_select_out.size(), 1);

          return masked_select_out;
      }


  
  

};

class MaxDim : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::max);
  }

  MaxDim(const torch::lazy::Value& self, const int64_t& dim, const bool& keepdim, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              MaxDim::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 2,
              torch::lazy::MHash(dim, keepdim)),
        dim(dim),
        keepdim(keepdim)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", keepdim=" << keepdim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim, const bool& keepdim) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("keepdim", keepdim);

          torch::lazy::TorchMlirOpVector max_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(max_out.size(), 2);

          return max_out;
      }


  int64_t dim;
  bool keepdim;
  

};

class Max : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::max);
  }

  Max(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Max::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector max_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(max_out.size(), 1);

      return max_out;
  }


  
  

};

class MaxPool2dWithIndices : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::max_pool2d_with_indices);
  }

  MaxPool2dWithIndices(const torch::lazy::Value& self, const ::std::vector<int64_t>& kernel_size, const ::std::vector<int64_t>& stride, const ::std::vector<int64_t>& padding, const ::std::vector<int64_t>& dilation, const bool& ceil_mode, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              MaxPool2dWithIndices::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 2,
              torch::lazy::MHash(kernel_size, stride, padding, dilation, ceil_mode)),
        kernel_size(kernel_size),
        stride(stride),
        padding(padding),
        dilation(dilation),
        ceil_mode(ceil_mode)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", kernel_size=" << kernel_size;
    ss << ", stride=" << stride;
    ss << ", padding=" << padding;
    ss << ", dilation=" << dilation;
    ss << ", ceil_mode=" << ceil_mode;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const ::std::vector<int64_t>& kernel_size, const ::std::vector<int64_t>& stride, const ::std::vector<int64_t>& padding, const ::std::vector<int64_t>& dilation, const bool& ceil_mode) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(6);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("kernel_size", kernel_size);
  arguments.emplace_back("stride", stride);
  arguments.emplace_back("padding", padding);
  arguments.emplace_back("dilation", dilation);
  arguments.emplace_back("ceil_mode", ceil_mode);

          torch::lazy::TorchMlirOpVector max_pool2d_with_indices_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(max_pool2d_with_indices_out.size(), 2);

          return max_pool2d_with_indices_out;
      }


  ::std::vector<int64_t> kernel_size;
  ::std::vector<int64_t> stride;
  ::std::vector<int64_t> padding;
  ::std::vector<int64_t> dilation;
  bool ceil_mode;
  

};

class MaxPool2dWithIndicesBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::max_pool2d_with_indices_backward);
  }

  MaxPool2dWithIndicesBackward(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const ::std::vector<int64_t>& kernel_size, const ::std::vector<int64_t>& stride, const ::std::vector<int64_t>& padding, const ::std::vector<int64_t>& dilation, const bool& ceil_mode, const torch::lazy::Value& indices, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              MaxPool2dWithIndicesBackward::ClassOpKind(),
              OpList{grad_output, self, indices},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(kernel_size, stride, padding, dilation, ceil_mode)),
        kernel_size(kernel_size),
        stride(stride),
        padding(padding),
        dilation(dilation),
        ceil_mode(ceil_mode)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", kernel_size=" << kernel_size;
    ss << ", stride=" << stride;
    ss << ", padding=" << padding;
    ss << ", dilation=" << dilation;
    ss << ", ceil_mode=" << ceil_mode;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const ::std::vector<int64_t>& kernel_size, const ::std::vector<int64_t>& stride, const ::std::vector<int64_t>& padding, const ::std::vector<int64_t>& dilation, const bool& ceil_mode, const torch::lazy::Value& indices) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(8);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("kernel_size", kernel_size);
  arguments.emplace_back("stride", stride);
  arguments.emplace_back("padding", padding);
  arguments.emplace_back("dilation", dilation);
  arguments.emplace_back("ceil_mode", ceil_mode);
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector max_pool2d_with_indices_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(max_pool2d_with_indices_backward_out.size(), 1);

          return max_pool2d_with_indices_backward_out;
      }


  ::std::vector<int64_t> kernel_size;
  ::std::vector<int64_t> stride;
  ::std::vector<int64_t> padding;
  ::std::vector<int64_t> dilation;
  bool ceil_mode;
  

};

class MaxPool3dWithIndices : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::max_pool3d_with_indices);
  }

  MaxPool3dWithIndices(const torch::lazy::Value& self, const ::std::vector<int64_t>& kernel_size, const ::std::vector<int64_t>& stride, const ::std::vector<int64_t>& padding, const ::std::vector<int64_t>& dilation, const bool& ceil_mode, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              MaxPool3dWithIndices::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 2,
              torch::lazy::MHash(kernel_size, stride, padding, dilation, ceil_mode)),
        kernel_size(kernel_size),
        stride(stride),
        padding(padding),
        dilation(dilation),
        ceil_mode(ceil_mode)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", kernel_size=" << kernel_size;
    ss << ", stride=" << stride;
    ss << ", padding=" << padding;
    ss << ", dilation=" << dilation;
    ss << ", ceil_mode=" << ceil_mode;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const ::std::vector<int64_t>& kernel_size, const ::std::vector<int64_t>& stride, const ::std::vector<int64_t>& padding, const ::std::vector<int64_t>& dilation, const bool& ceil_mode) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(6);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("kernel_size", kernel_size);
  arguments.emplace_back("stride", stride);
  arguments.emplace_back("padding", padding);
  arguments.emplace_back("dilation", dilation);
  arguments.emplace_back("ceil_mode", ceil_mode);

          torch::lazy::TorchMlirOpVector max_pool3d_with_indices_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(max_pool3d_with_indices_out.size(), 2);

          return max_pool3d_with_indices_out;
      }


  ::std::vector<int64_t> kernel_size;
  ::std::vector<int64_t> stride;
  ::std::vector<int64_t> padding;
  ::std::vector<int64_t> dilation;
  bool ceil_mode;
  

};

class MaxPool3dWithIndicesBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::max_pool3d_with_indices_backward);
  }

  MaxPool3dWithIndicesBackward(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const ::std::vector<int64_t>& kernel_size, const ::std::vector<int64_t>& stride, const ::std::vector<int64_t>& padding, const ::std::vector<int64_t>& dilation, const bool& ceil_mode, const torch::lazy::Value& indices, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              MaxPool3dWithIndicesBackward::ClassOpKind(),
              OpList{grad_output, self, indices},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(kernel_size, stride, padding, dilation, ceil_mode)),
        kernel_size(kernel_size),
        stride(stride),
        padding(padding),
        dilation(dilation),
        ceil_mode(ceil_mode)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", kernel_size=" << kernel_size;
    ss << ", stride=" << stride;
    ss << ", padding=" << padding;
    ss << ", dilation=" << dilation;
    ss << ", ceil_mode=" << ceil_mode;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const ::std::vector<int64_t>& kernel_size, const ::std::vector<int64_t>& stride, const ::std::vector<int64_t>& padding, const ::std::vector<int64_t>& dilation, const bool& ceil_mode, const torch::lazy::Value& indices) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(8);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("kernel_size", kernel_size);
  arguments.emplace_back("stride", stride);
  arguments.emplace_back("padding", padding);
  arguments.emplace_back("dilation", dilation);
  arguments.emplace_back("ceil_mode", ceil_mode);
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector max_pool3d_with_indices_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(max_pool3d_with_indices_backward_out.size(), 1);

          return max_pool3d_with_indices_backward_out;
      }


  ::std::vector<int64_t> kernel_size;
  ::std::vector<int64_t> stride;
  ::std::vector<int64_t> padding;
  ::std::vector<int64_t> dilation;
  bool ceil_mode;
  

};

class Maximum : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::maximum);
  }

  Maximum(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Maximum::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector maximum_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(maximum_out.size(), 1);

          return maximum_out;
      }


  
  

};

class Mean : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::mean);
  }

  Mean(const torch::lazy::Value& self, const c10::optional<at::ScalarType>& dtype, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Mean::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dtype)),
        dtype(dtype)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    if (dtype.has_value()) {
      ss << ", dtype=" << dtype.value();
    } else {
      ss << ", dtype=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const c10::optional<at::ScalarType>& dtype) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(1);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
      kwarguments.emplace_back("dtype", dtype);
      torch::lazy::TorchMlirOpVector mean_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(mean_out.size(), 1);

      return mean_out;
  }


  c10::optional<at::ScalarType> dtype;
  

};

class MeanDim : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::mean);
  }

  MeanDim(const torch::lazy::Value& self, const c10::optional<::std::vector<int64_t>>& dim, const bool& keepdim, const c10::optional<at::ScalarType>& dtype, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              MeanDim::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, keepdim, dtype)),
        dim(dim),
        keepdim(keepdim),
        dtype(dtype)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    if (dim.has_value()) {
      ss << ", dim=" << dim.value();
    } else {
      ss << ", dim=null";
    }
    ss << ", keepdim=" << keepdim;
    if (dtype.has_value()) {
      ss << ", dtype=" << dtype.value();
    } else {
      ss << ", dtype=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const c10::optional<::std::vector<int64_t>>& dim, const bool& keepdim, const c10::optional<at::ScalarType>& dtype) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(1);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("keepdim", keepdim);
          kwarguments.emplace_back("dtype", dtype);
          torch::lazy::TorchMlirOpVector mean_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(mean_out.size(), 1);

          return mean_out;
      }


  c10::optional<::std::vector<int64_t>> dim;
  bool keepdim;
  c10::optional<at::ScalarType> dtype;
  

};

class MinDim : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::min);
  }

  MinDim(const torch::lazy::Value& self, const int64_t& dim, const bool& keepdim, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              MinDim::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 2,
              torch::lazy::MHash(dim, keepdim)),
        dim(dim),
        keepdim(keepdim)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", keepdim=" << keepdim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim, const bool& keepdim) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("keepdim", keepdim);

          torch::lazy::TorchMlirOpVector min_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(min_out.size(), 2);

          return min_out;
      }


  int64_t dim;
  bool keepdim;
  

};

class Min : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::min);
  }

  Min(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Min::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector min_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(min_out.size(), 1);

      return min_out;
  }


  
  

};

class Minimum : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::minimum);
  }

  Minimum(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Minimum::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector minimum_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(minimum_out.size(), 1);

          return minimum_out;
      }


  
  

};

class Mish : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::mish);
  }

  Mish(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Mish::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector mish_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(mish_out.size(), 1);

      return mish_out;
  }


  
  

};

class Mm : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::mm);
  }

  Mm(const torch::lazy::Value& self, const torch::lazy::Value& mat2, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Mm::ClassOpKind(),
              OpList{self, mat2},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& mat2) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector mm_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(mm_out.size(), 1);

          return mm_out;
      }


  
  

};

class MseLoss : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::mse_loss);
  }

  MseLoss(const torch::lazy::Value& self, const torch::lazy::Value& target, const int64_t& reduction, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              MseLoss::ClassOpKind(),
              OpList{self, target},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(reduction)),
        reduction(reduction)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", reduction=" << reduction;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& target, const int64_t& reduction) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("reduction", reduction);

          torch::lazy::TorchMlirOpVector mse_loss_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(mse_loss_out.size(), 1);

          return mse_loss_out;
      }


  int64_t reduction;
  

};

class MseLossBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::mse_loss_backward);
  }

  MseLossBackward(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const torch::lazy::Value& target, const int64_t& reduction, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              MseLossBackward::ClassOpKind(),
              OpList{grad_output, self, target},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(reduction)),
        reduction(reduction)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", reduction=" << reduction;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const torch::lazy::Value& target, const int64_t& reduction) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("reduction", reduction);

          torch::lazy::TorchMlirOpVector mse_loss_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(mse_loss_backward_out.size(), 1);

          return mse_loss_backward_out;
      }


  int64_t reduction;
  

};

class MulTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::mul);
  }

  MulTensor(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              MulTensor::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector mul_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(mul_out.size(), 1);

          return mul_out;
      }


  
  

};

class MulScalar : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::mul);
  }

  MulScalar(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              MulScalar::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector mul_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(mul_out.size(), 1);

          return mul_out;
      }


  
  

};

class Multinomial : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::multinomial);
  }

  Multinomial(const torch::lazy::Value& self, const int64_t& num_samples, const bool& replacement, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Multinomial::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(num_samples, replacement)),
        num_samples(num_samples),
        replacement(replacement)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", num_samples=" << num_samples;
    ss << ", replacement=" << replacement;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& num_samples, const bool& replacement) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("num_samples", num_samples);
  arguments.emplace_back("replacement", replacement);

          torch::lazy::TorchMlirOpVector multinomial_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(multinomial_out.size(), 1);

          return multinomial_out;
      }


  int64_t num_samples;
  bool replacement;
  

};

class Mv : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::mv);
  }

  Mv(const torch::lazy::Value& self, const torch::lazy::Value& vec, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Mv::ClassOpKind(),
              OpList{self, vec},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& vec) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector mv_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(mv_out.size(), 1);

          return mv_out;
      }


  
  

};

class NativeBatchNorm : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::native_batch_norm);
  }

  NativeBatchNorm(const torch::lazy::Value& input, const c10::optional<torch::lazy::Value>& weight, const c10::optional<torch::lazy::Value>& bias, const c10::optional<torch::lazy::Value>& running_mean, const c10::optional<torch::lazy::Value>& running_var, const bool& training, const double& momentum, const double& eps, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              NativeBatchNorm::ClassOpKind(),
              OpList{input, weight.value_or(kNullValue), bias.value_or(kNullValue), running_mean.value_or(kNullValue), running_var.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 3,
              torch::lazy::MHash(training, momentum, eps)),
        training(training),
        momentum(momentum),
        eps(eps)
  {
    has_weight = !!weight;
    has_bias = !!bias;
    has_running_mean = !!running_mean;
    has_running_var = !!running_var;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", training=" << training;
    ss << ", momentum=" << momentum;
    ss << ", eps=" << eps;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& input, const c10::optional<torch::lazy::Value>& weight, const c10::optional<torch::lazy::Value>& bias, const c10::optional<torch::lazy::Value>& running_mean, const c10::optional<torch::lazy::Value>& running_var, const bool& training, const double& momentum, const double& eps) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(8);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(has_weight ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back(has_bias ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back(has_running_mean ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back(has_running_var ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back("training", training);
  arguments.emplace_back("momentum", momentum);
  arguments.emplace_back("eps", eps);

          torch::lazy::TorchMlirOpVector native_batch_norm_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(native_batch_norm_out.size(), 3);

          return native_batch_norm_out;
      }


  bool training;
  double momentum;
  double eps;
  bool has_weight: 1;
  bool has_bias: 1;
  bool has_running_mean: 1;
  bool has_running_var: 1;

};

class NativeBatchNormBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::native_batch_norm_backward);
  }

  NativeBatchNormBackward(const torch::lazy::Value& grad_out, const torch::lazy::Value& input, const c10::optional<torch::lazy::Value>& weight, const c10::optional<torch::lazy::Value>& running_mean, const c10::optional<torch::lazy::Value>& running_var, const c10::optional<torch::lazy::Value>& save_mean, const c10::optional<torch::lazy::Value>& save_invstd, const bool& train, const double& eps, const ::std::vector<bool>& output_mask, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              NativeBatchNormBackward::ClassOpKind(),
              OpList{grad_out, input, weight.value_or(kNullValue), running_mean.value_or(kNullValue), running_var.value_or(kNullValue), save_mean.value_or(kNullValue), save_invstd.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 3,
              torch::lazy::MHash(train, eps, output_mask)),
        train(train),
        eps(eps),
        output_mask(output_mask)
  {
    has_weight = !!weight;
    has_running_mean = !!running_mean;
    has_running_var = !!running_var;
    has_save_mean = !!save_mean;
    has_save_invstd = !!save_invstd;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", train=" << train;
    ss << ", eps=" << eps;
    ss << ", output_mask=" << output_mask;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_out, const torch::lazy::Value& input, const c10::optional<torch::lazy::Value>& weight, const c10::optional<torch::lazy::Value>& running_mean, const c10::optional<torch::lazy::Value>& running_var, const c10::optional<torch::lazy::Value>& save_mean, const c10::optional<torch::lazy::Value>& save_invstd, const bool& train, const double& eps, const ::std::vector<bool>& output_mask) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(10);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(has_weight ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back(has_running_mean ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back(has_running_var ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back(has_save_mean ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back(has_save_invstd ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back("train", train);
  arguments.emplace_back("eps", eps);
  arguments.emplace_back("output_mask", output_mask);

          torch::lazy::TorchMlirOpVector native_batch_norm_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(native_batch_norm_backward_out.size(), 3);

          return native_batch_norm_backward_out;
      }


  bool train;
  double eps;
  ::std::vector<bool> output_mask;
  bool has_weight: 1;
  bool has_running_mean: 1;
  bool has_running_var: 1;
  bool has_save_mean: 1;
  bool has_save_invstd: 1;

};

class NativeDropout : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::native_dropout);
  }

  NativeDropout(const torch::lazy::Value& input, const double& p, const c10::optional<bool>& train, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              NativeDropout::ClassOpKind(),
              OpList{input},
              std::move(shapes),
              /* num_outputs */ 2,
              torch::lazy::MHash(p, train)),
        p(p),
        train(train)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", p=" << p;
    if (train.has_value()) {
      ss << ", train=" << train.value();
    } else {
      ss << ", train=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& input, const double& p, const c10::optional<bool>& train) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("p", p);
  arguments.emplace_back("train", train);

          torch::lazy::TorchMlirOpVector native_dropout_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(native_dropout_out.size(), 2);

          return native_dropout_out;
      }


  double p;
  c10::optional<bool> train;
  

};

class NativeDropoutBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::native_dropout_backward);
  }

  NativeDropoutBackward(const torch::lazy::Value& grad_output, const torch::lazy::Value& mask, const double& scale, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              NativeDropoutBackward::ClassOpKind(),
              OpList{grad_output, mask},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(scale)),
        scale(scale)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", scale=" << scale;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& mask, const double& scale) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("scale", scale);

          torch::lazy::TorchMlirOpVector native_dropout_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(native_dropout_backward_out.size(), 1);

          return native_dropout_backward_out;
      }


  double scale;
  

};

class NativeGroupNorm : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::native_group_norm);
  }

  NativeGroupNorm(const torch::lazy::Value& input, const c10::optional<torch::lazy::Value>& weight, const c10::optional<torch::lazy::Value>& bias, const int64_t& N, const int64_t& C, const int64_t& HxW, const int64_t& group, const double& eps, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              NativeGroupNorm::ClassOpKind(),
              OpList{input, weight.value_or(kNullValue), bias.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 3,
              torch::lazy::MHash(N, C, HxW, group, eps)),
        N(N),
        C(C),
        HxW(HxW),
        group(group),
        eps(eps)
  {
    has_weight = !!weight;
    has_bias = !!bias;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", N=" << N;
    ss << ", C=" << C;
    ss << ", HxW=" << HxW;
    ss << ", group=" << group;
    ss << ", eps=" << eps;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& input, const c10::optional<torch::lazy::Value>& weight, const c10::optional<torch::lazy::Value>& bias, const int64_t& N, const int64_t& C, const int64_t& HxW, const int64_t& group, const double& eps) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(8);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(has_weight ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back(has_bias ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back("N", N);
  arguments.emplace_back("C", C);
  arguments.emplace_back("HxW", HxW);
  arguments.emplace_back("group", group);
  arguments.emplace_back("eps", eps);

          torch::lazy::TorchMlirOpVector native_group_norm_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(native_group_norm_out.size(), 3);

          return native_group_norm_out;
      }


  int64_t N;
  int64_t C;
  int64_t HxW;
  int64_t group;
  double eps;
  bool has_weight: 1;
  bool has_bias: 1;

};

class NativeGroupNormBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::native_group_norm_backward);
  }

  NativeGroupNormBackward(const torch::lazy::Value& grad_out, const torch::lazy::Value& input, const torch::lazy::Value& mean, const torch::lazy::Value& rstd, const c10::optional<torch::lazy::Value>& weight, const int64_t& N, const int64_t& C, const int64_t& HxW, const int64_t& group, const ::std::vector<bool>& output_mask, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              NativeGroupNormBackward::ClassOpKind(),
              OpList{grad_out, input, mean, rstd, weight.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 3,
              torch::lazy::MHash(N, C, HxW, group, output_mask)),
        N(N),
        C(C),
        HxW(HxW),
        group(group),
        output_mask(output_mask)
  {
    has_weight = !!weight;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", N=" << N;
    ss << ", C=" << C;
    ss << ", HxW=" << HxW;
    ss << ", group=" << group;
    ss << ", output_mask=" << output_mask;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_out, const torch::lazy::Value& input, const torch::lazy::Value& mean, const torch::lazy::Value& rstd, const c10::optional<torch::lazy::Value>& weight, const int64_t& N, const int64_t& C, const int64_t& HxW, const int64_t& group, const ::std::vector<bool>& output_mask) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(10);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(has_weight ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back("N", N);
  arguments.emplace_back("C", C);
  arguments.emplace_back("HxW", HxW);
  arguments.emplace_back("group", group);
  arguments.emplace_back("output_mask", output_mask);

          torch::lazy::TorchMlirOpVector native_group_norm_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(native_group_norm_backward_out.size(), 3);

          return native_group_norm_backward_out;
      }


  int64_t N;
  int64_t C;
  int64_t HxW;
  int64_t group;
  ::std::vector<bool> output_mask;
  bool has_weight: 1;

};

class NativeLayerNorm : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::native_layer_norm);
  }

  NativeLayerNorm(const torch::lazy::Value& input, const ::std::vector<int64_t>& normalized_shape, const c10::optional<torch::lazy::Value>& weight, const c10::optional<torch::lazy::Value>& bias, const double& eps, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              NativeLayerNorm::ClassOpKind(),
              OpList{input, weight.value_or(kNullValue), bias.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 3,
              torch::lazy::MHash(normalized_shape, eps)),
        normalized_shape(normalized_shape),
        eps(eps)
  {
    has_weight = !!weight;
    has_bias = !!bias;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", normalized_shape=" << normalized_shape;
    ss << ", eps=" << eps;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& input, const ::std::vector<int64_t>& normalized_shape, const c10::optional<torch::lazy::Value>& weight, const c10::optional<torch::lazy::Value>& bias, const double& eps) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(5);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("normalized_shape", normalized_shape);
  arguments.emplace_back(has_weight ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back(has_bias ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back("eps", eps);

          torch::lazy::TorchMlirOpVector native_layer_norm_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(native_layer_norm_out.size(), 3);

          return native_layer_norm_out;
      }


  ::std::vector<int64_t> normalized_shape;
  double eps;
  bool has_weight: 1;
  bool has_bias: 1;

};

class NativeLayerNormBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::native_layer_norm_backward);
  }

  NativeLayerNormBackward(const torch::lazy::Value& grad_out, const torch::lazy::Value& input, const ::std::vector<int64_t>& normalized_shape, const torch::lazy::Value& mean, const torch::lazy::Value& rstd, const c10::optional<torch::lazy::Value>& weight, const c10::optional<torch::lazy::Value>& bias, const ::std::vector<bool>& output_mask, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              NativeLayerNormBackward::ClassOpKind(),
              OpList{grad_out, input, mean, rstd, weight.value_or(kNullValue), bias.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 3,
              torch::lazy::MHash(normalized_shape, output_mask)),
        normalized_shape(normalized_shape),
        output_mask(output_mask)
  {
    has_weight = !!weight;
    has_bias = !!bias;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", normalized_shape=" << normalized_shape;
    ss << ", output_mask=" << output_mask;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_out, const torch::lazy::Value& input, const ::std::vector<int64_t>& normalized_shape, const torch::lazy::Value& mean, const torch::lazy::Value& rstd, const c10::optional<torch::lazy::Value>& weight, const c10::optional<torch::lazy::Value>& bias, const ::std::vector<bool>& output_mask) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(8);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("normalized_shape", normalized_shape);
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(has_weight ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back(has_bias ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back("output_mask", output_mask);

          torch::lazy::TorchMlirOpVector native_layer_norm_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(native_layer_norm_backward_out.size(), 3);

          return native_layer_norm_backward_out;
      }


  ::std::vector<int64_t> normalized_shape;
  ::std::vector<bool> output_mask;
  bool has_weight: 1;
  bool has_bias: 1;

};

class NeScalar : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::ne);
  }

  NeScalar(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              NeScalar::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector ne_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(ne_out.size(), 1);

          return ne_out;
      }


  
  

};

class NeTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::ne);
  }

  NeTensor(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              NeTensor::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector ne_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(ne_out.size(), 1);

          return ne_out;
      }


  
  

};

class Neg : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::neg);
  }

  Neg(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Neg::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector neg_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(neg_out.size(), 1);

      return neg_out;
  }


  
  

};

class NllLoss2dBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::nll_loss2d_backward);
  }

  NllLoss2dBackward(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const torch::lazy::Value& target, const c10::optional<torch::lazy::Value>& weight, const int64_t& reduction, const int64_t& ignore_index, const torch::lazy::Value& total_weight, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              NllLoss2dBackward::ClassOpKind(),
              OpList{grad_output, self, target, weight.value_or(kNullValue), total_weight},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(reduction, ignore_index)),
        reduction(reduction),
        ignore_index(ignore_index)
  {
    has_weight = !!weight;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", reduction=" << reduction;
    ss << ", ignore_index=" << ignore_index;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const torch::lazy::Value& target, const c10::optional<torch::lazy::Value>& weight, const int64_t& reduction, const int64_t& ignore_index, const torch::lazy::Value& total_weight) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(7);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(has_weight ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back("reduction", reduction);
  arguments.emplace_back("ignore_index", ignore_index);
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector nll_loss2d_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(nll_loss2d_backward_out.size(), 1);

          return nll_loss2d_backward_out;
      }


  int64_t reduction;
  int64_t ignore_index;
  bool has_weight: 1;

};

class NllLoss2dForward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::nll_loss2d_forward);
  }

  NllLoss2dForward(const torch::lazy::Value& self, const torch::lazy::Value& target, const c10::optional<torch::lazy::Value>& weight, const int64_t& reduction, const int64_t& ignore_index, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              NllLoss2dForward::ClassOpKind(),
              OpList{self, target, weight.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 2,
              torch::lazy::MHash(reduction, ignore_index)),
        reduction(reduction),
        ignore_index(ignore_index)
  {
    has_weight = !!weight;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", reduction=" << reduction;
    ss << ", ignore_index=" << ignore_index;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& target, const c10::optional<torch::lazy::Value>& weight, const int64_t& reduction, const int64_t& ignore_index) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(5);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(has_weight ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back("reduction", reduction);
  arguments.emplace_back("ignore_index", ignore_index);

          torch::lazy::TorchMlirOpVector nll_loss2d_forward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(nll_loss2d_forward_out.size(), 2);

          return nll_loss2d_forward_out;
      }


  int64_t reduction;
  int64_t ignore_index;
  bool has_weight: 1;

};

class NllLossBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::nll_loss_backward);
  }

  NllLossBackward(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const torch::lazy::Value& target, const c10::optional<torch::lazy::Value>& weight, const int64_t& reduction, const int64_t& ignore_index, const torch::lazy::Value& total_weight, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              NllLossBackward::ClassOpKind(),
              OpList{grad_output, self, target, weight.value_or(kNullValue), total_weight},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(reduction, ignore_index)),
        reduction(reduction),
        ignore_index(ignore_index)
  {
    has_weight = !!weight;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", reduction=" << reduction;
    ss << ", ignore_index=" << ignore_index;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const torch::lazy::Value& target, const c10::optional<torch::lazy::Value>& weight, const int64_t& reduction, const int64_t& ignore_index, const torch::lazy::Value& total_weight) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(7);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(has_weight ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back("reduction", reduction);
  arguments.emplace_back("ignore_index", ignore_index);
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector nll_loss_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(nll_loss_backward_out.size(), 1);

          return nll_loss_backward_out;
      }


  int64_t reduction;
  int64_t ignore_index;
  bool has_weight: 1;

};

class NllLossForward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::nll_loss_forward);
  }

  NllLossForward(const torch::lazy::Value& self, const torch::lazy::Value& target, const c10::optional<torch::lazy::Value>& weight, const int64_t& reduction, const int64_t& ignore_index, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              NllLossForward::ClassOpKind(),
              OpList{self, target, weight.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 2,
              torch::lazy::MHash(reduction, ignore_index)),
        reduction(reduction),
        ignore_index(ignore_index)
  {
    has_weight = !!weight;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", reduction=" << reduction;
    ss << ", ignore_index=" << ignore_index;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& target, const c10::optional<torch::lazy::Value>& weight, const int64_t& reduction, const int64_t& ignore_index) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(5);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(has_weight ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back("reduction", reduction);
  arguments.emplace_back("ignore_index", ignore_index);

          torch::lazy::TorchMlirOpVector nll_loss_forward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(nll_loss_forward_out.size(), 2);

          return nll_loss_forward_out;
      }


  int64_t reduction;
  int64_t ignore_index;
  bool has_weight: 1;

};

class Nonzero : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::nonzero);
  }

  Nonzero(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Nonzero::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector nonzero_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(nonzero_out.size(), 1);

      return nonzero_out;
  }


  
  

};

class NonzeroStatic : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::nonzero_static);
  }

  NonzeroStatic(const torch::lazy::Value& self, const int64_t& size, const int64_t& fill_value, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              NonzeroStatic::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(size, fill_value)),
        size(size),
        fill_value(fill_value)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", size=" << size;
    ss << ", fill_value=" << fill_value;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& size, const int64_t& fill_value) const {
    return false;
    }

  
          TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
              PRINT_FUNCTION();
              std::vector<torch::jit::NamedValue> arguments;
              std::vector<torch::jit::NamedValue> kwarguments;
              arguments.reserve(1);
              kwarguments.reserve(2);
              size_t i = 0;
              arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
              kwarguments.emplace_back("size", size);
  kwarguments.emplace_back("fill_value", fill_value);
              torch::lazy::TorchMlirOpVector nonzero_static_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
              TORCH_CHECK_EQ(nonzero_static_out.size(), 1);

              return nonzero_static_out;
          }


  int64_t size;
  int64_t fill_value;
  

};

class NormScalaroptDim : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::norm);
  }

  NormScalaroptDim(const torch::lazy::Value& self, const c10::optional<torch::lazy::Value>& p, const ::std::vector<int64_t>& dim, const bool& keepdim, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              NormScalaroptDim::ClassOpKind(),
              OpList{self, p.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, keepdim)),
        dim(dim),
        keepdim(keepdim)
  {
    has_p = !!p;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", keepdim=" << keepdim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const c10::optional<torch::lazy::Value>& p, const ::std::vector<int64_t>& dim, const bool& keepdim) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(has_p ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("keepdim", keepdim);

          torch::lazy::TorchMlirOpVector norm_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(norm_out.size(), 1);

          return norm_out;
      }


  ::std::vector<int64_t> dim;
  bool keepdim;
  bool has_p: 1;

};

class NormalFunctional : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::normal_functional);
  }

  NormalFunctional(const torch::lazy::Value& self, const double& mean, const double& std, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              NormalFunctional::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(mean, std)),
        mean(mean),
        std(std)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", mean=" << mean;
    ss << ", std=" << std;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const double& mean, const double& std) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("mean", mean);
  arguments.emplace_back("std", std);

          torch::lazy::TorchMlirOpVector normal_functional_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(normal_functional_out.size(), 1);

          return normal_functional_out;
      }


  double mean;
  double std;
  

};

class PermuteCopy : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::permute_copy);
  }

  PermuteCopy(const torch::lazy::Value& self, const ::std::vector<int64_t>& dims, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              PermuteCopy::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dims)),
        dims(dims)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dims=" << dims;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const ::std::vector<int64_t>& dims) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dims", dims);

          torch::lazy::TorchMlirOpVector permute_copy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(permute_copy_out.size(), 1);

          return permute_copy_out;
      }


  ::std::vector<int64_t> dims;
  

};

class PowTensorTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::pow);
  }

  PowTensorTensor(const torch::lazy::Value& self, const torch::lazy::Value& exponent, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              PowTensorTensor::ClassOpKind(),
              OpList{self, exponent},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& exponent) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector pow_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(pow_out.size(), 1);

          return pow_out;
      }


  
  

};

class PowScalar : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::pow);
  }

  PowScalar(const torch::lazy::Value& self, const torch::lazy::Value& exponent, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              PowScalar::ClassOpKind(),
              OpList{self, exponent},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& exponent) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector pow_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(pow_out.size(), 1);

          return pow_out;
      }


  
  

};

class PowTensorScalar : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::pow);
  }

  PowTensorScalar(const torch::lazy::Value& self, const torch::lazy::Value& exponent, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              PowTensorScalar::ClassOpKind(),
              OpList{self, exponent},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& exponent) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector pow_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(pow_out.size(), 1);

          return pow_out;
      }


  
  

};

class ProdDimInt : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::prod);
  }

  ProdDimInt(const torch::lazy::Value& self, const int64_t& dim, const bool& keepdim, const c10::optional<at::ScalarType>& dtype, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ProdDimInt::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, keepdim, dtype)),
        dim(dim),
        keepdim(keepdim),
        dtype(dtype)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", keepdim=" << keepdim;
    if (dtype.has_value()) {
      ss << ", dtype=" << dtype.value();
    } else {
      ss << ", dtype=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim, const bool& keepdim, const c10::optional<at::ScalarType>& dtype) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(1);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("keepdim", keepdim);
          kwarguments.emplace_back("dtype", dtype);
          torch::lazy::TorchMlirOpVector prod_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(prod_out.size(), 1);

          return prod_out;
      }


  int64_t dim;
  bool keepdim;
  c10::optional<at::ScalarType> dtype;
  

};

class Rand : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::rand);
  }

  Rand(const ::std::vector<int64_t>& size, const c10::optional<at::ScalarType>& dtype, const c10::optional<at::Layout>& layout, const c10::optional<at::Device>& device, const c10::optional<bool>& pin_memory, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Rand::ClassOpKind(),
              OpList{},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(size, dtype, layout, device, pin_memory)),
        size(size),
        dtype(dtype),
        layout(layout),
        device(device),
        pin_memory(pin_memory)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", size=" << size;
    if (dtype.has_value()) {
      ss << ", dtype=" << dtype.value();
    } else {
      ss << ", dtype=null";
    }
    if (layout.has_value()) {
      ss << ", layout=" << layout.value();
    } else {
      ss << ", layout=null";
    }
    if (device.has_value()) {
      ss << ", device=" << device.value();
    } else {
      ss << ", device=null";
    }
    if (pin_memory.has_value()) {
      ss << ", pin_memory=" << pin_memory.value();
    } else {
      ss << ", pin_memory=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const ::std::vector<int64_t>& size, const c10::optional<at::ScalarType>& dtype, const c10::optional<at::Layout>& layout, const c10::optional<at::Device>& device, const c10::optional<bool>& pin_memory) const {
    return false;
    }

  
          TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
              PRINT_FUNCTION();
              std::vector<torch::jit::NamedValue> arguments;
              std::vector<torch::jit::NamedValue> kwarguments;
              arguments.reserve(1);
              kwarguments.reserve(4);

              arguments.emplace_back("size", size);
              kwarguments.emplace_back("dtype", dtype);
  kwarguments.emplace_back("layout", layout);
  kwarguments.emplace_back("device", device);
  kwarguments.emplace_back("pin_memory", pin_memory);
              torch::lazy::TorchMlirOpVector rand_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
              TORCH_CHECK_EQ(rand_out.size(), 1);

              return rand_out;
          }


  ::std::vector<int64_t> size;
  c10::optional<at::ScalarType> dtype;
  c10::optional<at::Layout> layout;
  c10::optional<at::Device> device;
  c10::optional<bool> pin_memory;
  

};

class Randint : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::randint);
  }

  Randint(const int64_t& high, const ::std::vector<int64_t>& size, const c10::optional<at::ScalarType>& dtype, const c10::optional<at::Layout>& layout, const c10::optional<at::Device>& device, const c10::optional<bool>& pin_memory, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Randint::ClassOpKind(),
              OpList{},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(high, size, dtype, layout, device, pin_memory)),
        high(high),
        size(size),
        dtype(dtype),
        layout(layout),
        device(device),
        pin_memory(pin_memory)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", high=" << high;
    ss << ", size=" << size;
    if (dtype.has_value()) {
      ss << ", dtype=" << dtype.value();
    } else {
      ss << ", dtype=null";
    }
    if (layout.has_value()) {
      ss << ", layout=" << layout.value();
    } else {
      ss << ", layout=null";
    }
    if (device.has_value()) {
      ss << ", device=" << device.value();
    } else {
      ss << ", device=null";
    }
    if (pin_memory.has_value()) {
      ss << ", pin_memory=" << pin_memory.value();
    } else {
      ss << ", pin_memory=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const int64_t& high, const ::std::vector<int64_t>& size, const c10::optional<at::ScalarType>& dtype, const c10::optional<at::Layout>& layout, const c10::optional<at::Device>& device, const c10::optional<bool>& pin_memory) const {
    return false;
    }

  
          TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
              PRINT_FUNCTION();
              std::vector<torch::jit::NamedValue> arguments;
              std::vector<torch::jit::NamedValue> kwarguments;
              arguments.reserve(2);
              kwarguments.reserve(4);

              arguments.emplace_back("high", high);
      arguments.emplace_back("size", size);
              kwarguments.emplace_back("dtype", dtype);
  kwarguments.emplace_back("layout", layout);
  kwarguments.emplace_back("device", device);
  kwarguments.emplace_back("pin_memory", pin_memory);
              torch::lazy::TorchMlirOpVector randint_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
              TORCH_CHECK_EQ(randint_out.size(), 1);

              return randint_out;
          }


  int64_t high;
  ::std::vector<int64_t> size;
  c10::optional<at::ScalarType> dtype;
  c10::optional<at::Layout> layout;
  c10::optional<at::Device> device;
  c10::optional<bool> pin_memory;
  

};

class RandintLow : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::randint);
  }

  RandintLow(const int64_t& low, const int64_t& high, const ::std::vector<int64_t>& size, const c10::optional<at::ScalarType>& dtype, const c10::optional<at::Layout>& layout, const c10::optional<at::Device>& device, const c10::optional<bool>& pin_memory, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              RandintLow::ClassOpKind(),
              OpList{},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(low, high, size, dtype, layout, device, pin_memory)),
        low(low),
        high(high),
        size(size),
        dtype(dtype),
        layout(layout),
        device(device),
        pin_memory(pin_memory)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", low=" << low;
    ss << ", high=" << high;
    ss << ", size=" << size;
    if (dtype.has_value()) {
      ss << ", dtype=" << dtype.value();
    } else {
      ss << ", dtype=null";
    }
    if (layout.has_value()) {
      ss << ", layout=" << layout.value();
    } else {
      ss << ", layout=null";
    }
    if (device.has_value()) {
      ss << ", device=" << device.value();
    } else {
      ss << ", device=null";
    }
    if (pin_memory.has_value()) {
      ss << ", pin_memory=" << pin_memory.value();
    } else {
      ss << ", pin_memory=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const int64_t& low, const int64_t& high, const ::std::vector<int64_t>& size, const c10::optional<at::ScalarType>& dtype, const c10::optional<at::Layout>& layout, const c10::optional<at::Device>& device, const c10::optional<bool>& pin_memory) const {
    return false;
    }

  
          TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
              PRINT_FUNCTION();
              std::vector<torch::jit::NamedValue> arguments;
              std::vector<torch::jit::NamedValue> kwarguments;
              arguments.reserve(3);
              kwarguments.reserve(4);

              arguments.emplace_back("low", low);
      arguments.emplace_back("high", high);
      arguments.emplace_back("size", size);
              kwarguments.emplace_back("dtype", dtype);
  kwarguments.emplace_back("layout", layout);
  kwarguments.emplace_back("device", device);
  kwarguments.emplace_back("pin_memory", pin_memory);
              torch::lazy::TorchMlirOpVector randint_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
              TORCH_CHECK_EQ(randint_out.size(), 1);

              return randint_out;
          }


  int64_t low;
  int64_t high;
  ::std::vector<int64_t> size;
  c10::optional<at::ScalarType> dtype;
  c10::optional<at::Layout> layout;
  c10::optional<at::Device> device;
  c10::optional<bool> pin_memory;
  

};

class Randn : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::randn);
  }

  Randn(const ::std::vector<int64_t>& size, const c10::optional<at::ScalarType>& dtype, const c10::optional<at::Layout>& layout, const c10::optional<at::Device>& device, const c10::optional<bool>& pin_memory, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Randn::ClassOpKind(),
              OpList{},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(size, dtype, layout, device, pin_memory)),
        size(size),
        dtype(dtype),
        layout(layout),
        device(device),
        pin_memory(pin_memory)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", size=" << size;
    if (dtype.has_value()) {
      ss << ", dtype=" << dtype.value();
    } else {
      ss << ", dtype=null";
    }
    if (layout.has_value()) {
      ss << ", layout=" << layout.value();
    } else {
      ss << ", layout=null";
    }
    if (device.has_value()) {
      ss << ", device=" << device.value();
    } else {
      ss << ", device=null";
    }
    if (pin_memory.has_value()) {
      ss << ", pin_memory=" << pin_memory.value();
    } else {
      ss << ", pin_memory=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const ::std::vector<int64_t>& size, const c10::optional<at::ScalarType>& dtype, const c10::optional<at::Layout>& layout, const c10::optional<at::Device>& device, const c10::optional<bool>& pin_memory) const {
    return false;
    }

  
          TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
              PRINT_FUNCTION();
              std::vector<torch::jit::NamedValue> arguments;
              std::vector<torch::jit::NamedValue> kwarguments;
              arguments.reserve(1);
              kwarguments.reserve(4);

              arguments.emplace_back("size", size);
              kwarguments.emplace_back("dtype", dtype);
  kwarguments.emplace_back("layout", layout);
  kwarguments.emplace_back("device", device);
  kwarguments.emplace_back("pin_memory", pin_memory);
              torch::lazy::TorchMlirOpVector randn_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
              TORCH_CHECK_EQ(randn_out.size(), 1);

              return randn_out;
          }


  ::std::vector<int64_t> size;
  c10::optional<at::ScalarType> dtype;
  c10::optional<at::Layout> layout;
  c10::optional<at::Device> device;
  c10::optional<bool> pin_memory;
  

};

class RandnGenerator : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::randn);
  }

  RandnGenerator(const ::std::vector<int64_t>& size, const c10::optional<at::ScalarType>& dtype, const c10::optional<at::Layout>& layout, const c10::optional<at::Device>& device, const c10::optional<bool>& pin_memory, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              RandnGenerator::ClassOpKind(),
              OpList{},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(size, dtype, layout, device, pin_memory)),
        size(size),
        dtype(dtype),
        layout(layout),
        device(device),
        pin_memory(pin_memory)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", size=" << size;
    if (dtype.has_value()) {
      ss << ", dtype=" << dtype.value();
    } else {
      ss << ", dtype=null";
    }
    if (layout.has_value()) {
      ss << ", layout=" << layout.value();
    } else {
      ss << ", layout=null";
    }
    if (device.has_value()) {
      ss << ", device=" << device.value();
    } else {
      ss << ", device=null";
    }
    if (pin_memory.has_value()) {
      ss << ", pin_memory=" << pin_memory.value();
    } else {
      ss << ", pin_memory=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const ::std::vector<int64_t>& size, const c10::optional<at::ScalarType>& dtype, const c10::optional<at::Layout>& layout, const c10::optional<at::Device>& device, const c10::optional<bool>& pin_memory) const {
    return false;
    }

  
          TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
              PRINT_FUNCTION();
              std::vector<torch::jit::NamedValue> arguments;
              std::vector<torch::jit::NamedValue> kwarguments;
              arguments.reserve(1);
              kwarguments.reserve(4);

              arguments.emplace_back("size", size);
              kwarguments.emplace_back("dtype", dtype);
  kwarguments.emplace_back("layout", layout);
  kwarguments.emplace_back("device", device);
  kwarguments.emplace_back("pin_memory", pin_memory);
              torch::lazy::TorchMlirOpVector randn_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
              TORCH_CHECK_EQ(randn_out.size(), 1);

              return randn_out;
          }


  ::std::vector<int64_t> size;
  c10::optional<at::ScalarType> dtype;
  c10::optional<at::Layout> layout;
  c10::optional<at::Device> device;
  c10::optional<bool> pin_memory;
  

};

class Reciprocal : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::reciprocal);
  }

  Reciprocal(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Reciprocal::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector reciprocal_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(reciprocal_out.size(), 1);

      return reciprocal_out;
  }


  
  

};

class Relu : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::relu);
  }

  Relu(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Relu::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector relu_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(relu_out.size(), 1);

      return relu_out;
  }


  
  

};

class RemainderScalar : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::remainder);
  }

  RemainderScalar(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              RemainderScalar::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector remainder_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(remainder_out.size(), 1);

          return remainder_out;
      }


  
  

};

class RemainderTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::remainder);
  }

  RemainderTensor(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              RemainderTensor::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector remainder_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(remainder_out.size(), 1);

          return remainder_out;
      }


  
  

};

class Repeat : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::repeat);
  }

  Repeat(const torch::lazy::Value& self, const ::std::vector<int64_t>& repeats, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Repeat::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(repeats)),
        repeats(repeats)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", repeats=" << repeats;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const ::std::vector<int64_t>& repeats) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("repeats", repeats);

          torch::lazy::TorchMlirOpVector repeat_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(repeat_out.size(), 1);

          return repeat_out;
      }


  ::std::vector<int64_t> repeats;
  

};

class Roll : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::roll);
  }

  Roll(const torch::lazy::Value& self, const ::std::vector<int64_t>& shifts, const ::std::vector<int64_t>& dims, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Roll::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(shifts, dims)),
        shifts(shifts),
        dims(dims)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", shifts=" << shifts;
    ss << ", dims=" << dims;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const ::std::vector<int64_t>& shifts, const ::std::vector<int64_t>& dims) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("shifts", shifts);
  arguments.emplace_back("dims", dims);

          torch::lazy::TorchMlirOpVector roll_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(roll_out.size(), 1);

          return roll_out;
      }


  ::std::vector<int64_t> shifts;
  ::std::vector<int64_t> dims;
  

};

class Round : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::round);
  }

  Round(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Round::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector round_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(round_out.size(), 1);

      return round_out;
  }


  
  

};

class Rsqrt : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::rsqrt);
  }

  Rsqrt(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Rsqrt::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector rsqrt_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(rsqrt_out.size(), 1);

      return rsqrt_out;
  }


  
  

};

class ScatterSrc : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::scatter);
  }

  ScatterSrc(const torch::lazy::Value& self, const int64_t& dim, const torch::lazy::Value& index, const torch::lazy::Value& src, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ScatterSrc::ClassOpKind(),
              OpList{self, index, src},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim)),
        dim(dim)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim, const torch::lazy::Value& index, const torch::lazy::Value& src) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector scatter_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(scatter_out.size(), 1);

          return scatter_out;
      }


  int64_t dim;
  

};

class ScatterValue : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::scatter);
  }

  ScatterValue(const torch::lazy::Value& self, const int64_t& dim, const torch::lazy::Value& index, const torch::lazy::Value& value, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ScatterValue::ClassOpKind(),
              OpList{self, index, value},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim)),
        dim(dim)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim, const torch::lazy::Value& index, const torch::lazy::Value& value) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector scatter_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(scatter_out.size(), 1);

          return scatter_out;
      }


  int64_t dim;
  

};

class ScatterReduce : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::scatter);
  }

  ScatterReduce(const torch::lazy::Value& self, const int64_t& dim, const torch::lazy::Value& index, const torch::lazy::Value& src, const c10::string_view& reduce, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ScatterReduce::ClassOpKind(),
              OpList{self, index, src},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, reduce)),
        dim(dim),
        reduce(reduce)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", reduce=" << reduce;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim, const torch::lazy::Value& index, const torch::lazy::Value& src, const c10::string_view& reduce) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(1);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
          kwarguments.emplace_back("reduce", reduce);
          torch::lazy::TorchMlirOpVector scatter_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(scatter_out.size(), 1);

          return scatter_out;
      }


  int64_t dim;
  std::string reduce;
  

};

class ScatterAdd : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::scatter_add);
  }

  ScatterAdd(const torch::lazy::Value& self, const int64_t& dim, const torch::lazy::Value& index, const torch::lazy::Value& src, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ScatterAdd::ClassOpKind(),
              OpList{self, index, src},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim)),
        dim(dim)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim, const torch::lazy::Value& index, const torch::lazy::Value& src) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector scatter_add_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(scatter_add_out.size(), 1);

          return scatter_add_out;
      }


  int64_t dim;
  

};

class ScatterReduceTwo : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::scatter_reduce);
  }

  ScatterReduceTwo(const torch::lazy::Value& self, const int64_t& dim, const torch::lazy::Value& index, const torch::lazy::Value& src, const c10::string_view& reduce, const bool& include_self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ScatterReduceTwo::ClassOpKind(),
              OpList{self, index, src},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, reduce, include_self)),
        dim(dim),
        reduce(reduce),
        include_self(include_self)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", reduce=" << reduce;
    ss << ", include_self=" << include_self;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim, const torch::lazy::Value& index, const torch::lazy::Value& src, const c10::string_view& reduce, const bool& include_self) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(5);
          kwarguments.reserve(1);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("reduce", reduce);
          kwarguments.emplace_back("include_self", include_self);
          torch::lazy::TorchMlirOpVector scatter_reduce_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(scatter_reduce_out.size(), 1);

          return scatter_reduce_out;
      }


  int64_t dim;
  std::string reduce;
  bool include_self;
  

};

class SelectCopyInt : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::select_copy);
  }

  SelectCopyInt(const torch::lazy::Value& self, const int64_t& dim, const int64_t& index, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              SelectCopyInt::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, index)),
        dim(dim),
        index(index)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", index=" << index;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim, const int64_t& index) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("index", index);

          torch::lazy::TorchMlirOpVector select_copy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(select_copy_out.size(), 1);

          return select_copy_out;
      }


  int64_t dim;
  int64_t index;
  

};

class SelectScatter : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::select_scatter);
  }

  SelectScatter(const torch::lazy::Value& self, const torch::lazy::Value& src, const int64_t& dim, const int64_t& index, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              SelectScatter::ClassOpKind(),
              OpList{self, src},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, index)),
        dim(dim),
        index(index)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", index=" << index;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& src, const int64_t& dim, const int64_t& index) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("index", index);

          torch::lazy::TorchMlirOpVector select_scatter_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(select_scatter_out.size(), 1);

          return select_scatter_out;
      }


  int64_t dim;
  int64_t index;
  

};

class Sgn : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::sgn);
  }

  Sgn(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Sgn::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector sgn_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(sgn_out.size(), 1);

      return sgn_out;
  }


  
  

};

class Sigmoid : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::sigmoid);
  }

  Sigmoid(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Sigmoid::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector sigmoid_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(sigmoid_out.size(), 1);

      return sigmoid_out;
  }


  
  

};

class SigmoidBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(c10::Symbol::fromQualString("aten::sigmoid_backward"));
  }

  SigmoidBackward(const torch::lazy::Value& grad_output, const torch::lazy::Value& output, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              SigmoidBackward::ClassOpKind(),
              OpList{grad_output, output},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& output) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector sigmoid_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(sigmoid_backward_out.size(), 1);

          return sigmoid_backward_out;
      }


  
  

};

class Sign : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::sign);
  }

  Sign(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Sign::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector sign_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(sign_out.size(), 1);

      return sign_out;
  }


  
  

};

class Silu : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::silu);
  }

  Silu(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Silu::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector silu_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(silu_out.size(), 1);

      return silu_out;
  }


  
  

};

class Sin : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::sin);
  }

  Sin(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Sin::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector sin_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(sin_out.size(), 1);

      return sin_out;
  }


  
  

};

class SliceCopyTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::slice_copy);
  }

  SliceCopyTensor(const torch::lazy::Value& self, const int64_t& dim, const c10::optional<torch::lazy::Value>& start, const c10::optional<torch::lazy::Value>& end, const torch::lazy::Value& step, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              SliceCopyTensor::ClassOpKind(),
              OpList{self, start.value_or(kNullValue), end.value_or(kNullValue), step},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim)),
        dim(dim)
  {
    has_start = !!start;
    has_end = !!end;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim, const c10::optional<torch::lazy::Value>& start, const c10::optional<torch::lazy::Value>& end, const torch::lazy::Value& step) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(5);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back(has_start ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back(has_end ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector slice_copy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(slice_copy_out.size(), 1);

          return slice_copy_out;
      }


  int64_t dim;
  bool has_start: 1;
  bool has_end: 1;

};

class SliceScatter : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::slice_scatter);
  }

  SliceScatter(const torch::lazy::Value& self, const torch::lazy::Value& src, const int64_t& dim, const c10::optional<torch::lazy::Value>& start, const c10::optional<torch::lazy::Value>& end, const torch::lazy::Value& step, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              SliceScatter::ClassOpKind(),
              OpList{self, src, start.value_or(kNullValue), end.value_or(kNullValue), step},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim)),
        dim(dim)
  {
    has_start = !!start;
    has_end = !!end;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& src, const int64_t& dim, const c10::optional<torch::lazy::Value>& start, const c10::optional<torch::lazy::Value>& end, const torch::lazy::Value& step) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(6);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back(has_start ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back(has_end ? loctx->GetOutputOp(operand(i++)) : nullptr);
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector slice_scatter_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(slice_scatter_out.size(), 1);

          return slice_scatter_out;
      }


  int64_t dim;
  bool has_start: 1;
  bool has_end: 1;

};

class Softplus : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::softplus);
  }

  Softplus(const torch::lazy::Value& self, const torch::lazy::Value& beta, const torch::lazy::Value& threshold, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Softplus::ClassOpKind(),
              OpList{self, beta, threshold},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& beta, const torch::lazy::Value& threshold) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector softplus_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(softplus_out.size(), 1);

          return softplus_out;
      }


  
  

};

class Sort : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::sort);
  }

  Sort(const torch::lazy::Value& self, const int64_t& dim, const bool& descending, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Sort::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 2,
              torch::lazy::MHash(dim, descending)),
        dim(dim),
        descending(descending)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    ss << ", descending=" << descending;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim, const bool& descending) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("descending", descending);

          torch::lazy::TorchMlirOpVector sort_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(sort_out.size(), 2);

          return sort_out;
      }


  int64_t dim;
  bool descending;
  

};

class Sqrt : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::sqrt);
  }

  Sqrt(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Sqrt::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector sqrt_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(sqrt_out.size(), 1);

      return sqrt_out;
  }


  
  

};

class SqueezeCopy : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::squeeze_copy);
  }

  SqueezeCopy(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              SqueezeCopy::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector squeeze_copy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(squeeze_copy_out.size(), 1);

      return squeeze_copy_out;
  }


  
  

};

class SqueezeCopyDim : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::squeeze_copy);
  }

  SqueezeCopyDim(const torch::lazy::Value& self, const int64_t& dim, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              SqueezeCopyDim::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim)),
        dim(dim)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);

          torch::lazy::TorchMlirOpVector squeeze_copy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(squeeze_copy_out.size(), 1);

          return squeeze_copy_out;
      }


  int64_t dim;
  

};

class Stack : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::stack);
  }

  Stack(const torch::lazy::Value& tensors, const int64_t& dim, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Stack::ClassOpKind(),
              OpList{tensors},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim)),
        dim(dim)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& tensors, const int64_t& dim) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);

          torch::lazy::TorchMlirOpVector stack_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(stack_out.size(), 1);

          return stack_out;
      }


  int64_t dim;
  

};

class StdCorrection : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::std);
  }

  StdCorrection(const torch::lazy::Value& self, const c10::optional<::std::vector<int64_t>>& dim, const c10::optional<torch::lazy::Value>& correction, const bool& keepdim, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              StdCorrection::ClassOpKind(),
              OpList{self, correction.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, keepdim)),
        dim(dim),
        keepdim(keepdim)
  {
    has_correction = !!correction;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    if (dim.has_value()) {
      ss << ", dim=" << dim.value();
    } else {
      ss << ", dim=null";
    }
    ss << ", keepdim=" << keepdim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const c10::optional<::std::vector<int64_t>>& dim, const c10::optional<torch::lazy::Value>& correction, const bool& keepdim) const {
    return false;
    }

  
          TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
              PRINT_FUNCTION();
              std::vector<torch::jit::NamedValue> arguments;
              std::vector<torch::jit::NamedValue> kwarguments;
              arguments.reserve(2);
              kwarguments.reserve(2);
              size_t i = 0;
              arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
      arguments.emplace_back("dim", dim);
              kwarguments.emplace_back("correction", loctx->GetOutputOp(operand(i++)));
  kwarguments.emplace_back("keepdim", keepdim);
              torch::lazy::TorchMlirOpVector std_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
              TORCH_CHECK_EQ(std_out.size(), 1);

              return std_out;
          }


  c10::optional<::std::vector<int64_t>> dim;
  bool keepdim;
  bool has_correction: 1;

};

class SubTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::sub);
  }

  SubTensor(const torch::lazy::Value& self, const torch::lazy::Value& other, const torch::lazy::Value& alpha, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              SubTensor::ClassOpKind(),
              OpList{self, other, alpha},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other, const torch::lazy::Value& alpha) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(1);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
          kwarguments.emplace_back("alpha", loctx->GetOutputOp(operand(i++)));
          torch::lazy::TorchMlirOpVector sub_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(sub_out.size(), 1);

          return sub_out;
      }


  
  

};

class SubScalar : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::sub);
  }

  SubScalar(const torch::lazy::Value& self, const torch::lazy::Value& other, const torch::lazy::Value& alpha, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              SubScalar::ClassOpKind(),
              OpList{self, other, alpha},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other, const torch::lazy::Value& alpha) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector sub_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(sub_out.size(), 1);

          return sub_out;
      }


  
  

};

class Sum : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::sum);
  }

  Sum(const torch::lazy::Value& self, const c10::optional<at::ScalarType>& dtype, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Sum::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dtype)),
        dtype(dtype)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    if (dtype.has_value()) {
      ss << ", dtype=" << dtype.value();
    } else {
      ss << ", dtype=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const c10::optional<at::ScalarType>& dtype) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(1);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
      kwarguments.emplace_back("dtype", dtype);
      torch::lazy::TorchMlirOpVector sum_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(sum_out.size(), 1);

      return sum_out;
  }


  c10::optional<at::ScalarType> dtype;
  

};

class SumDimIntlist : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::sum);
  }

  SumDimIntlist(const torch::lazy::Value& self, const c10::optional<::std::vector<int64_t>>& dim, const bool& keepdim, const c10::optional<at::ScalarType>& dtype, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              SumDimIntlist::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, keepdim, dtype)),
        dim(dim),
        keepdim(keepdim),
        dtype(dtype)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    if (dim.has_value()) {
      ss << ", dim=" << dim.value();
    } else {
      ss << ", dim=null";
    }
    ss << ", keepdim=" << keepdim;
    if (dtype.has_value()) {
      ss << ", dtype=" << dtype.value();
    } else {
      ss << ", dtype=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const c10::optional<::std::vector<int64_t>>& dim, const bool& keepdim, const c10::optional<at::ScalarType>& dtype) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(1);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("keepdim", keepdim);
          kwarguments.emplace_back("dtype", dtype);
          torch::lazy::TorchMlirOpVector sum_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(sum_out.size(), 1);

          return sum_out;
      }


  c10::optional<::std::vector<int64_t>> dim;
  bool keepdim;
  c10::optional<at::ScalarType> dtype;
  

};

class TCopy : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::t_copy);
  }

  TCopy(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              TCopy::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector t_copy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(t_copy_out.size(), 1);

      return t_copy_out;
  }


  
  

};

class Tanh : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::tanh);
  }

  Tanh(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Tanh::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector tanh_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(tanh_out.size(), 1);

      return tanh_out;
  }


  
  

};

class TanhBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::tanh_backward);
  }

  TanhBackward(const torch::lazy::Value& grad_output, const torch::lazy::Value& output, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              TanhBackward::ClassOpKind(),
              OpList{grad_output, output},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& output) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector tanh_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(tanh_backward_out.size(), 1);

          return tanh_backward_out;
      }


  
  

};

class Threshold : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::threshold);
  }

  Threshold(const torch::lazy::Value& self, const torch::lazy::Value& threshold, const torch::lazy::Value& value, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Threshold::ClassOpKind(),
              OpList{self, threshold, value},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& threshold, const torch::lazy::Value& value) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector threshold_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(threshold_out.size(), 1);

          return threshold_out;
      }


  
  

};

class ThresholdBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::threshold_backward);
  }

  ThresholdBackward(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const torch::lazy::Value& threshold, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ThresholdBackward::ClassOpKind(),
              OpList{grad_output, self, threshold},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const torch::lazy::Value& self, const torch::lazy::Value& threshold) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector threshold_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(threshold_backward_out.size(), 1);

          return threshold_backward_out;
      }


  
  

};

class Topk : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::topk);
  }

  Topk(const torch::lazy::Value& self, const int64_t& k, const int64_t& dim, const bool& largest, const bool& sorted, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Topk::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 2,
              torch::lazy::MHash(k, dim, largest, sorted)),
        k(k),
        dim(dim),
        largest(largest),
        sorted(sorted)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", k=" << k;
    ss << ", dim=" << dim;
    ss << ", largest=" << largest;
    ss << ", sorted=" << sorted;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& k, const int64_t& dim, const bool& largest, const bool& sorted) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(5);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("k", k);
  arguments.emplace_back("dim", dim);
  arguments.emplace_back("largest", largest);
  arguments.emplace_back("sorted", sorted);

          torch::lazy::TorchMlirOpVector topk_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(topk_out.size(), 2);

          return topk_out;
      }


  int64_t k;
  int64_t dim;
  bool largest;
  bool sorted;
  

};

class TransposeCopyInt : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::transpose_copy);
  }

  TransposeCopyInt(const torch::lazy::Value& self, const int64_t& dim0, const int64_t& dim1, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              TransposeCopyInt::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim0, dim1)),
        dim0(dim0),
        dim1(dim1)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim0=" << dim0;
    ss << ", dim1=" << dim1;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim0, const int64_t& dim1) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim0", dim0);
  arguments.emplace_back("dim1", dim1);

          torch::lazy::TorchMlirOpVector transpose_copy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(transpose_copy_out.size(), 1);

          return transpose_copy_out;
      }


  int64_t dim0;
  int64_t dim1;
  

};

class Tril : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::tril);
  }

  Tril(const torch::lazy::Value& self, const int64_t& diagonal, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Tril::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(diagonal)),
        diagonal(diagonal)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", diagonal=" << diagonal;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& diagonal) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("diagonal", diagonal);

          torch::lazy::TorchMlirOpVector tril_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(tril_out.size(), 1);

          return tril_out;
      }


  int64_t diagonal;
  

};

class Triu : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::triu);
  }

  Triu(const torch::lazy::Value& self, const int64_t& diagonal, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Triu::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(diagonal)),
        diagonal(diagonal)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", diagonal=" << diagonal;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& diagonal) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("diagonal", diagonal);

          torch::lazy::TorchMlirOpVector triu_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(triu_out.size(), 1);

          return triu_out;
      }


  int64_t diagonal;
  

};

class UnfoldCopy : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::unfold_copy);
  }

  UnfoldCopy(const torch::lazy::Value& self, const int64_t& dimension, const int64_t& size, const int64_t& step, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              UnfoldCopy::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dimension, size, step)),
        dimension(dimension),
        size(size),
        step(step)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dimension=" << dimension;
    ss << ", size=" << size;
    ss << ", step=" << step;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dimension, const int64_t& size, const int64_t& step) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dimension", dimension);
  arguments.emplace_back("size", size);
  arguments.emplace_back("step", step);

          torch::lazy::TorchMlirOpVector unfold_copy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(unfold_copy_out.size(), 1);

          return unfold_copy_out;
      }


  int64_t dimension;
  int64_t size;
  int64_t step;
  

};

class Uniform : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::uniform);
  }

  Uniform(const torch::lazy::Value& self, const double& from, const double& to, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Uniform::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(from, to)),
        from(from),
        to(to)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", from=" << from;
    ss << ", to=" << to;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const double& from, const double& to) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("from", from);
  arguments.emplace_back("to", to);

          torch::lazy::TorchMlirOpVector uniform_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(uniform_out.size(), 1);

          return uniform_out;
      }


  double from;
  double to;
  

};

class UniqueConsecutive : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::unique_consecutive);
  }

  UniqueConsecutive(const torch::lazy::Value& self, const bool& return_inverse, const bool& return_counts, const c10::optional<int64_t>& dim, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              UniqueConsecutive::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 3,
              torch::lazy::MHash(return_inverse, return_counts, dim)),
        return_inverse(return_inverse),
        return_counts(return_counts),
        dim(dim)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", return_inverse=" << return_inverse;
    ss << ", return_counts=" << return_counts;
    if (dim.has_value()) {
      ss << ", dim=" << dim.value();
    } else {
      ss << ", dim=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const bool& return_inverse, const bool& return_counts, const c10::optional<int64_t>& dim) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("return_inverse", return_inverse);
  arguments.emplace_back("return_counts", return_counts);
  arguments.emplace_back("dim", dim);

          torch::lazy::TorchMlirOpVector unique_consecutive_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(unique_consecutive_out.size(), 3);

          return unique_consecutive_out;
      }


  bool return_inverse;
  bool return_counts;
  c10::optional<int64_t> dim;
  

};

class UnsqueezeCopy : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::unsqueeze_copy);
  }

  UnsqueezeCopy(const torch::lazy::Value& self, const int64_t& dim, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              UnsqueezeCopy::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim)),
        dim(dim)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dim=" << dim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const int64_t& dim) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dim", dim);

          torch::lazy::TorchMlirOpVector unsqueeze_copy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(unsqueeze_copy_out.size(), 1);

          return unsqueeze_copy_out;
      }


  int64_t dim;
  

};

class UpsampleNearest2d : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::upsample_nearest2d);
  }

  UpsampleNearest2d(const torch::lazy::Value& self, const ::std::vector<int64_t>& output_size, const c10::optional<double>& scales_h, const c10::optional<double>& scales_w, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              UpsampleNearest2d::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(output_size, scales_h, scales_w)),
        output_size(output_size),
        scales_h(scales_h),
        scales_w(scales_w)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", output_size=" << output_size;
    if (scales_h.has_value()) {
      ss << ", scales_h=" << scales_h.value();
    } else {
      ss << ", scales_h=null";
    }
    if (scales_w.has_value()) {
      ss << ", scales_w=" << scales_w.value();
    } else {
      ss << ", scales_w=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const ::std::vector<int64_t>& output_size, const c10::optional<double>& scales_h, const c10::optional<double>& scales_w) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(4);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("output_size", output_size);
  arguments.emplace_back("scales_h", scales_h);
  arguments.emplace_back("scales_w", scales_w);

          torch::lazy::TorchMlirOpVector upsample_nearest2d_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(upsample_nearest2d_out.size(), 1);

          return upsample_nearest2d_out;
      }


  ::std::vector<int64_t> output_size;
  c10::optional<double> scales_h;
  c10::optional<double> scales_w;
  

};

class UpsampleNearest2dBackward : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::upsample_nearest2d_backward);
  }

  UpsampleNearest2dBackward(const torch::lazy::Value& grad_output, const ::std::vector<int64_t>& output_size, const ::std::vector<int64_t>& input_size, const c10::optional<double>& scales_h, const c10::optional<double>& scales_w, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              UpsampleNearest2dBackward::ClassOpKind(),
              OpList{grad_output},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(output_size, input_size, scales_h, scales_w)),
        output_size(output_size),
        input_size(input_size),
        scales_h(scales_h),
        scales_w(scales_w)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", output_size=" << output_size;
    ss << ", input_size=" << input_size;
    if (scales_h.has_value()) {
      ss << ", scales_h=" << scales_h.value();
    } else {
      ss << ", scales_h=null";
    }
    if (scales_w.has_value()) {
      ss << ", scales_w=" << scales_w.value();
    } else {
      ss << ", scales_w=null";
    }
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& grad_output, const ::std::vector<int64_t>& output_size, const ::std::vector<int64_t>& input_size, const c10::optional<double>& scales_h, const c10::optional<double>& scales_w) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(5);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("output_size", output_size);
  arguments.emplace_back("input_size", input_size);
  arguments.emplace_back("scales_h", scales_h);
  arguments.emplace_back("scales_w", scales_w);

          torch::lazy::TorchMlirOpVector upsample_nearest2d_backward_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(upsample_nearest2d_backward_out.size(), 1);

          return upsample_nearest2d_backward_out;
      }


  ::std::vector<int64_t> output_size;
  ::std::vector<int64_t> input_size;
  c10::optional<double> scales_h;
  c10::optional<double> scales_w;
  

};

class VarCorrection : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::var);
  }

  VarCorrection(const torch::lazy::Value& self, const c10::optional<::std::vector<int64_t>>& dim, const c10::optional<torch::lazy::Value>& correction, const bool& keepdim, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              VarCorrection::ClassOpKind(),
              OpList{self, correction.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dim, keepdim)),
        dim(dim),
        keepdim(keepdim)
  {
    has_correction = !!correction;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    if (dim.has_value()) {
      ss << ", dim=" << dim.value();
    } else {
      ss << ", dim=null";
    }
    ss << ", keepdim=" << keepdim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const c10::optional<::std::vector<int64_t>>& dim, const c10::optional<torch::lazy::Value>& correction, const bool& keepdim) const {
    return false;
    }

  
          TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
              PRINT_FUNCTION();
              std::vector<torch::jit::NamedValue> arguments;
              std::vector<torch::jit::NamedValue> kwarguments;
              arguments.reserve(2);
              kwarguments.reserve(2);
              size_t i = 0;
              arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
      arguments.emplace_back("dim", dim);
              kwarguments.emplace_back("correction", loctx->GetOutputOp(operand(i++)));
  kwarguments.emplace_back("keepdim", keepdim);
              torch::lazy::TorchMlirOpVector var_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
              TORCH_CHECK_EQ(var_out.size(), 1);

              return var_out;
          }


  c10::optional<::std::vector<int64_t>> dim;
  bool keepdim;
  bool has_correction: 1;

};

class VarMeanCorrection : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::var_mean);
  }

  VarMeanCorrection(const torch::lazy::Value& self, const c10::optional<::std::vector<int64_t>>& dim, const c10::optional<torch::lazy::Value>& correction, const bool& keepdim, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              VarMeanCorrection::ClassOpKind(),
              OpList{self, correction.value_or(kNullValue)},
              std::move(shapes),
              /* num_outputs */ 2,
              torch::lazy::MHash(dim, keepdim)),
        dim(dim),
        keepdim(keepdim)
  {
    has_correction = !!correction;
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    if (dim.has_value()) {
      ss << ", dim=" << dim.value();
    } else {
      ss << ", dim=null";
    }
    ss << ", keepdim=" << keepdim;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const c10::optional<::std::vector<int64_t>>& dim, const c10::optional<torch::lazy::Value>& correction, const bool& keepdim) const {
    return false;
    }

  
          TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
              PRINT_FUNCTION();
              std::vector<torch::jit::NamedValue> arguments;
              std::vector<torch::jit::NamedValue> kwarguments;
              arguments.reserve(2);
              kwarguments.reserve(2);
              size_t i = 0;
              arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
      arguments.emplace_back("dim", dim);
              kwarguments.emplace_back("correction", loctx->GetOutputOp(operand(i++)));
  kwarguments.emplace_back("keepdim", keepdim);
              torch::lazy::TorchMlirOpVector var_mean_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
              TORCH_CHECK_EQ(var_mean_out.size(), 2);

              return var_mean_out;
          }


  c10::optional<::std::vector<int64_t>> dim;
  bool keepdim;
  bool has_correction: 1;

};

class ViewAsComplex : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::view_as_complex);
  }

  ViewAsComplex(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ViewAsComplex::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector view_as_complex_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(view_as_complex_out.size(), 1);

      return view_as_complex_out;
  }


  
  

};

class ViewAsReal : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::view_as_real);
  }

  ViewAsReal(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ViewAsReal::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector view_as_real_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(view_as_real_out.size(), 1);

      return view_as_real_out;
  }


  
  

};

class ViewCopy : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::view_copy);
  }

  ViewCopy(const torch::lazy::Value& self, const ::std::vector<int64_t>& size, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ViewCopy::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(size)),
        size(size)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", size=" << size;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const ::std::vector<int64_t>& size) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("size", size);

          torch::lazy::TorchMlirOpVector view_copy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(view_copy_out.size(), 1);

          return view_copy_out;
      }


  ::std::vector<int64_t> size;
  

};

class ViewCopyDtype : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::view_copy);
  }

  ViewCopyDtype(const torch::lazy::Value& self, const at::ScalarType& dtype, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              ViewCopyDtype::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash(dtype)),
        dtype(dtype)
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    ss << ", dtype=" << dtype;
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const at::ScalarType& dtype) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back("dtype", dtype);

          torch::lazy::TorchMlirOpVector view_copy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(view_copy_out.size(), 1);

          return view_copy_out;
      }


  at::ScalarType dtype;
  

};

class WhereSelf : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::where);
  }

  WhereSelf(const torch::lazy::Value& condition, const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              WhereSelf::ClassOpKind(),
              OpList{condition, self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& condition, const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(3);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector where_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(where_out.size(), 1);

          return where_out;
      }


  
  

};

class XlogyTensor : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::xlogy);
  }

  XlogyTensor(const torch::lazy::Value& self, const torch::lazy::Value& other, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              XlogyTensor::ClassOpKind(),
              OpList{self, other},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self, const torch::lazy::Value& other) const {
    return false;
    }

  
      TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
          PRINT_FUNCTION();
          std::vector<torch::jit::NamedValue> arguments;
          std::vector<torch::jit::NamedValue> kwarguments;
          arguments.reserve(2);
          kwarguments.reserve(0);
          size_t i = 0;
          arguments.emplace_back(loctx->GetOutputOp(operand(i++)));
  arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

          torch::lazy::TorchMlirOpVector xlogy_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
          TORCH_CHECK_EQ(xlogy_out.size(), 1);

          return xlogy_out;
      }


  
  

};

class Zero : public torch::lazy::TorchMlirNode {
 public:
  static torch::lazy::OpKind ClassOpKind() {
    return torch::lazy::OpKind(at::aten::zero);
  }

  Zero(const torch::lazy::Value& self, std::vector<torch::lazy::Shape>&& shapes)
      : torch::lazy::TorchMlirNode(
              Zero::ClassOpKind(),
              OpList{self},
              std::move(shapes),
              /* num_outputs */ 1,
              torch::lazy::MHash())
  {
    
  }

  std::string ToString() const override {
    std::stringstream ss;
    ss << torch::lazy::TorchMlirNode::ToString();
    
    return ss.str();
  }

  

  bool CanBeReused(const torch::lazy::Value& self) const {
    return false;
    }

  
  TorchMlirOpVector Lower(TorchMlirFunction function, TorchMlirLoweringContext* loctx) const override {
      PRINT_FUNCTION();
      std::vector<torch::jit::NamedValue> arguments;
      std::vector<torch::jit::NamedValue> kwarguments;
      arguments.reserve(1);
      kwarguments.reserve(0);
      size_t i = 0;
      arguments.emplace_back(loctx->GetOutputOp(operand(i++)));

      torch::lazy::TorchMlirOpVector zero_out = torch::lazy::LowerTorchMlirBuiltin(function, op().op, shapes(), arguments, kwarguments);
      TORCH_CHECK_EQ(zero_out.size(), 1);

      return zero_out;
  }


  
  

};

} // namespace lazy
} // namespace torch
